;;; This is the test program for the procedures introduced in the chapter
;;; on combinatorial constructions.

;;; John David Stone
;;; Department of Computer Science
;;; Grinnell College

;;; created October 25, 2011
;;; last revised January 6, 2017

(import (afp primitives)
        (afp arithmetic)
        (afp extras)
        (only (afp couplers) pipe)
        (only (afp predicate-operations) ^et ^vel)
        (only (afp pairs) pair=? pair-of pair-of=)
        (only (afp lists)
              empty-list? list=? list-of list-of= fold-list run position-in
              positional-weights separate-indices all-but-position)
        (only (afp sources)
              tap finite-source finite-source->list fold-finite-source
              ordered-prepend-to-each-source
              ordered-prepend-each-to-each-source)
        (only (afp bags)
              bag bag? put-into-bag empty-bag? bag=? bag-of bag-of=
              cons-with-each cons-with-each-source prepend-to-each
              prepend-each-to-each put-into-each-bag remove-bag-from-bag)
        (afp products-and-selections)
        (afp permutations)
        (afp partitions)
        (support testing)
        (support testing-procedures))

;;; Test the procedures from the "Cartesian products" section.

(suite cons-with-each ((right-type? (bag-of (pair-of symbol? number?)))
                       (equiv? (bag-of= (pair-of= symbol=? =))))
  (test empty-bag
    (cons-with-each 'yav (bag))
    1 (right-type?)
    empty-bag?)
  (test typical
    (cons-with-each 'ihtct (bag 247 221 776 751 659))
    1 (right-type?)
    (match? equiv? (bag (cons 'ihtct 247)
                        (cons 'ihtct 221)
                        (cons 'ihtct 776)
                        (cons 'ihtct 751)
                        (cons 'ihtct 659)))))

(suite Cartesian-product ()
  (test left-empty
    (Cartesian-product (bag) (bag 'lth 'dovn 'krh 'rar 'othciis))
    1 (bag?)
    empty-bag?)
  (test right-empty
    (Cartesian-product (bag 'brmm 'radtuos 'tttoy 'oioanor) (bag))
    1 (bag?)
    empty-bag?)
  (test non-empty-product
    (Cartesian-product (bag 'asti 'ecles 'pem 'ese)
                       (bag 'awekvh 'teadcle 'aenet))
    1 ((bag-of pair?))
    (match? (bag-of= pair=?) (bag (cons 'asti 'awekvh)
                                  (cons 'asti 'teadcle)
                                  (cons 'asti 'aenet)
                                  (cons 'ecles 'awekvh)
                                  (cons 'ecles 'teadcle)
                                  (cons 'ecles 'aenet)
                                  (cons 'pem 'awekvh)
                                  (cons 'pem 'teadcle)
                                  (cons 'pem 'aenet)
                                  (cons 'ese 'awekvh)
                                  (cons 'ese 'teadcle)
                                  (cons 'ese 'aenet)))))

(suite cons-with-each-source
    ((finite-source->bag (rec (bagger src)
                           (receive (initial others) (tap src)
                             (if (end-of-source? initial)
                                 (bag)
                                 (put-into-bag initial (bagger others))))))
     (equiv? (bag-of= (pair-of= symbol=? =))))
  (test empty-bag
    (cons-with-each-source 'yav (bag))
    1 (source?)
    (pipe finite-source->bag empty-bag?))
  (test typical
    (cons-with-each-source 'owenear (bag 935 854 57 674))
    1 (source?)
    (pipe finite-source->bag
          (match? equiv? (bag (cons 'owenear 935)
                              (cons 'owenear 854)
                              (cons 'owenear 57)
                              (cons 'owenear 674))))))

(suite Cartesian-product-source ()
  (test left-empty
    (Cartesian-product-source (bag) (bag 'nrthgel 'sstvw 'dli 'ilontti))
    1 (source?)
    empty-source?)
  (test right-empty
    (Cartesian-product-source (bag 'faryt 'cip 'mthd 'inoaut 'restnt 'sdcf)
                              (bag))
    1 (source?)
    empty-source?)
  (test non-empty-product
    (Cartesian-product-source (bag 'krnhro 'osraose)
                              (bag 'kronowm 'tieh 'idnl 'kbns 'eutorng))
    1 (source?)
    (pipe (fold-finite-source bag put-into-bag)
          (match? (bag-of= pair=?) (bag (cons 'krnhro 'kronowm)
                                        (cons 'krnhro 'tieh)
                                        (cons 'krnhro 'idnl)
                                        (cons 'krnhro 'kbns)
                                        (cons 'krnhro 'eutorng)
                                        (cons 'osraose 'kronowm)
                                        (cons 'osraose 'tieh)
                                        (cons 'osraose 'idnl)
                                        (cons 'osraose 'kbns)
                                        (cons 'osraose 'eutorng))))))

(suite prepend-to-each ((right-type? (bag-of (list-of symbol?)))
                        (equiv? (bag-of= (list-of= symbol=?))))
  (test empty-bag
    (prepend-to-each 'enan (bag))
    1 (right-type?)
    empty-bag?)
  (test singleton-bag
    (prepend-to-each 'nei (bag (list 'ytevmrp 'whiro 'tneue 'neo)))
    1 (right-type?)
    (match? equiv? (bag (list 'nei 'ytevmrp 'whiro 'tneue 'neo))))
  (test more-members
    (prepend-to-each 'aqlmelr (bag (list 'tolsti 'oap 'abhi 'oihr)
                                   (list 'dgnire)
                                   (list)
                                   (list 'pfehat 'ffbeti 'ode 'edndmer
                                         'afsa 'eenrti 'ven 'reisee
                                         'feaes)))
    1 (right-type?)
    (match? equiv? (bag (list 'aqlmelr 'tolsti 'oap 'abhi 'oihr) 
                        (list 'aqlmelr 'dgnire)
                        (list 'aqlmelr)
                        (list 'aqlmelr 'pfehat 'ffbeti 'ode 'edndmer 'afsa
                              'eenrti 'ven 'reisee 'feaes)))))

(suite prepend-each-to-each ((product-bag? (bag-of list?))
                             (product-bag=? (bag-of= list=?)))
  (test left-empty
    (prepend-each-to-each (bag) (bag (list 'uwt 'ruanhe)
                                     (list 'eof 'oedmfg)
                                     (list 'hitni 'ilb)))
    1 (product-bag?)
    empty-bag?)
  (test right-empty
    (prepend-each-to-each (bag 'drtev 'bes 'ogmttp 'lsl 'muyt) (bag))
    1 (product-bag?)
    empty-bag?)
  (test nontrivial
    (prepend-each-to-each (bag 'oote 'uarean 'sies)
                          (bag (list 'edtptu 'shn 'lpds 'trddue)
                               (list 'odka 'piytihr 'eshifot 'chtft)
                               (list 'ttooo 'asdffu 'tiom 'donyuh)))
    1 (product-bag?)
    (match? product-bag=?
            (bag (list 'oote 'edtptu 'shn 'lpds 'trddue)
                 (list 'oote 'odka 'piytihr 'eshifot 'chtft)
                 (list 'oote 'ttooo 'asdffu 'tiom 'donyuh)
                 (list 'uarean 'edtptu 'shn 'lpds 'trddue)
                 (list 'uarean 'odka 'piytihr 'eshifot 'chtft)
                 (list 'uarean 'ttooo 'asdffu 'tiom 'donyuh)
                 (list 'sies 'edtptu 'shn 'lpds 'trddue)
                 (list 'sies 'odka 'piytihr 'eshifot 'chtft)
                 (list 'sies 'ttooo 'asdffu 'tiom 'donyuh)))))

(suite generalized-Cartesian-product ((product-bag? (bag-of list?))
                                      (product-bag=? (bag-of= list=?)))
  (test no-arguments
    (generalized-Cartesian-product)
    1 (product-bag?)
    (match? product-bag=? (bag (list))))
  (test one-argument
    (generalized-Cartesian-product (bag 'ttsom 'snee 'onpgs))
    1 (product-bag?)
    (match? product-bag=? (bag (list 'ttsom) (list 'snee) (list 'onpgs))))
  (test several-arguments
    (generalized-Cartesian-product (bag 'hartei 'fhsctf)
                                   (bag 'hfnt 'otite 'iiwryea)
                                   (bag 'kygefp 'thn 'erd 'auiit)
                                   (bag 'vhs 'sluolg))
    1 (product-bag?)
    (match? product-bag=? (bag (list 'hartei 'hfnt 'kygefp 'vhs)
                               (list 'hartei 'hfnt 'kygefp 'sluolg)
                               (list 'hartei 'hfnt 'thn 'vhs)
                               (list 'hartei 'hfnt 'thn 'sluolg)
                               (list 'hartei 'hfnt 'erd 'vhs)
                               (list 'hartei 'hfnt 'erd 'sluolg)
                               (list 'hartei 'hfnt 'auiit 'vhs)
                               (list 'hartei 'hfnt 'auiit 'sluolg)
                               (list 'hartei 'otite 'kygefp 'vhs)
                               (list 'hartei 'otite 'kygefp 'sluolg)
                               (list 'hartei 'otite 'thn 'vhs)
                               (list 'hartei 'otite 'thn 'sluolg)
                               (list 'hartei 'otite 'erd 'vhs)
                               (list 'hartei 'otite 'erd 'sluolg)
                               (list 'hartei 'otite 'auiit 'vhs)
                               (list 'hartei 'otite 'auiit 'sluolg)
                               (list 'hartei 'iiwryea 'kygefp 'vhs)
                               (list 'hartei 'iiwryea 'kygefp 'sluolg)
                               (list 'hartei 'iiwryea 'thn 'vhs)
                               (list 'hartei 'iiwryea 'thn 'sluolg)
                               (list 'hartei 'iiwryea 'erd 'vhs)
                               (list 'hartei 'iiwryea 'erd 'sluolg)
                               (list 'hartei 'iiwryea 'auiit 'vhs)
                               (list 'hartei 'iiwryea 'auiit 'sluolg)
                               (list 'fhsctf 'hfnt 'kygefp 'vhs)
                               (list 'fhsctf 'hfnt 'kygefp 'sluolg)
                               (list 'fhsctf 'hfnt 'thn 'vhs)
                               (list 'fhsctf 'hfnt 'thn 'sluolg)
                               (list 'fhsctf 'hfnt 'erd 'vhs)
                               (list 'fhsctf 'hfnt 'erd 'sluolg)
                               (list 'fhsctf 'hfnt 'auiit 'vhs)
                               (list 'fhsctf 'hfnt 'auiit 'sluolg)
                               (list 'fhsctf 'otite 'kygefp 'vhs)
                               (list 'fhsctf 'otite 'kygefp 'sluolg)
                               (list 'fhsctf 'otite 'thn 'vhs)
                               (list 'fhsctf 'otite 'thn 'sluolg)
                               (list 'fhsctf 'otite 'erd 'vhs)
                               (list 'fhsctf 'otite 'erd 'sluolg)
                               (list 'fhsctf 'otite 'auiit 'vhs)
                               (list 'fhsctf 'otite 'auiit 'sluolg)
                               (list 'fhsctf 'iiwryea 'kygefp 'vhs)
                               (list 'fhsctf 'iiwryea 'kygefp 'sluolg)
                               (list 'fhsctf 'iiwryea 'thn 'vhs)
                               (list 'fhsctf 'iiwryea 'thn 'sluolg)
                               (list 'fhsctf 'iiwryea 'erd 'vhs)
                               (list 'fhsctf 'iiwryea 'erd 'sluolg)
                               (list 'fhsctf 'iiwryea 'auiit 'vhs)
                               (list 'fhsctf 'iiwryea 'auiit 'sluolg))))
  (test empty-bag
    (generalized-Cartesian-product (bag 'tcthe 'irgatol 'iloy 'rrns 'otlem)
                                   (bag 'ush 'eeloulc 'yntno)
                                   (bag)
                                   (bag 'eslp 'sopldst 'noahha)
                                   (bag 'ewo 'intsu 'ngenouo 'ehingo))
    1 (product-bag?)
    empty-bag?))                                   

(suite Cartesian-power ((power-bag? (bag-of list?))
                        (power-bag=? (bag-of= list=?)))
  (test minimal
    (Cartesian-power (bag) 0)
    1 (power-bag?)
    (match? power-bag=? (bag (list))))
  (test zeroth-power
    (Cartesian-power (bag 'ueetns 'rto 'rntduya 'otnosua 'seh 'raed 'gnlyfw
                          'hneccmn)
                     0)
    1 (power-bag?)
    (match? power-bag=? (bag (list))))
  (test empty-bag
    (Cartesian-power (bag) 364)
    1 (power-bag?)
    empty-bag?)
  (test singleton-bag
    (Cartesian-power (bag 'ueetns) 5)
    1 (power-bag?)
    (match? power-bag=?
            (bag (list 'ueetns 'ueetns 'ueetns 'ueetns 'ueetns))))
  (test typical-case
    (Cartesian-power (bag 'rto 'rntduya 'otnosua) 3)
    1 (power-bag?)
    (match? power-bag=? (bag (list 'rto 'rto 'rto)
                             (list 'rto 'rto 'rntduya)
                             (list 'rto 'rto 'otnosua)
                             (list 'rto 'rntduya 'rto)
                             (list 'rto 'rntduya 'rntduya)
                             (list 'rto 'rntduya 'otnosua)
                             (list 'rto 'otnosua 'rto)
                             (list 'rto 'otnosua 'rntduya)
                             (list 'rto 'otnosua 'otnosua)
                             (list 'rntduya 'rto 'rto)
                             (list 'rntduya 'rto 'rntduya)
                             (list 'rntduya 'rto 'otnosua)
                             (list 'rntduya 'rntduya 'rto)
                             (list 'rntduya 'rntduya 'rntduya)
                             (list 'rntduya 'rntduya 'otnosua)
                             (list 'rntduya 'otnosua 'rto)
                             (list 'rntduya 'otnosua 'rntduya)
                             (list 'rntduya 'otnosua 'otnosua)
                             (list 'otnosua 'rto 'rto)
                             (list 'otnosua 'rto 'rntduya)
                             (list 'otnosua 'rto 'otnosua)
                             (list 'otnosua 'rntduya 'rto)
                             (list 'otnosua 'rntduya 'rntduya)
                             (list 'otnosua 'rntduya 'otnosua)
                             (list 'otnosua 'otnosua 'rto)
                             (list 'otnosua 'otnosua 'rntduya)
                             (list 'otnosua 'otnosua 'otnosua))))
  (test duplicates
    (Cartesian-power (bag 'seh 'raed 'seh) 4)
    1 (power-bag?)
    (match? power-bag=? (bag (list 'seh 'seh 'seh 'seh)
                             (list 'seh 'seh 'seh 'raed)
                             (list 'seh 'seh 'seh 'seh)
                             (list 'seh 'seh 'raed 'seh)
                             (list 'seh 'seh 'raed 'raed)
                             (list 'seh 'seh 'raed 'seh)
                             (list 'seh 'seh 'seh 'seh)
                             (list 'seh 'seh 'seh 'raed)
                             (list 'seh 'seh 'seh 'seh)
                             (list 'seh 'raed 'seh 'seh)
                             (list 'seh 'raed 'seh 'raed)
                             (list 'seh 'raed 'seh 'seh)
                             (list 'seh 'raed 'raed 'seh)
                             (list 'seh 'raed 'raed 'raed)
                             (list 'seh 'raed 'raed 'seh)
                             (list 'seh 'raed 'seh 'seh)
                             (list 'seh 'raed 'seh 'raed)
                             (list 'seh 'raed 'seh 'seh)
                             (list 'seh 'seh 'seh 'seh)
                             (list 'seh 'seh 'seh 'raed)
                             (list 'seh 'seh 'seh 'seh)
                             (list 'seh 'seh 'raed 'seh)
                             (list 'seh 'seh 'raed 'raed)
                             (list 'seh 'seh 'raed 'seh)
                             (list 'seh 'seh 'seh 'seh)
                             (list 'seh 'seh 'seh 'raed)
                             (list 'seh 'seh 'seh 'seh)
                             (list 'raed 'seh 'seh 'seh)
                             (list 'raed 'seh 'seh 'raed)
                             (list 'raed 'seh 'seh 'seh)
                             (list 'raed 'seh 'raed 'seh)
                             (list 'raed 'seh 'raed 'raed)
                             (list 'raed 'seh 'raed 'seh)
                             (list 'raed 'seh 'seh 'seh)
                             (list 'raed 'seh 'seh 'raed)
                             (list 'raed 'seh 'seh 'seh)
                             (list 'raed 'raed 'seh 'seh)
                             (list 'raed 'raed 'seh 'raed)
                             (list 'raed 'raed 'seh 'seh)
                             (list 'raed 'raed 'raed 'seh)
                             (list 'raed 'raed 'raed 'raed)
                             (list 'raed 'raed 'raed 'seh)
                             (list 'raed 'raed 'seh 'seh)
                             (list 'raed 'raed 'seh 'raed)
                             (list 'raed 'raed 'seh 'seh)
                             (list 'raed 'seh 'seh 'seh)
                             (list 'raed 'seh 'seh 'raed)
                             (list 'raed 'seh 'seh 'seh)
                             (list 'raed 'seh 'raed 'seh)
                             (list 'raed 'seh 'raed 'raed)
                             (list 'raed 'seh 'raed 'seh)
                             (list 'raed 'seh 'seh 'seh)
                             (list 'raed 'seh 'seh 'raed)
                             (list 'raed 'seh 'seh 'seh)
                             (list 'seh 'seh 'seh 'seh)
                             (list 'seh 'seh 'seh 'raed)
                             (list 'seh 'seh 'seh 'seh)
                             (list 'seh 'seh 'raed 'seh)
                             (list 'seh 'seh 'raed 'raed)
                             (list 'seh 'seh 'raed 'seh)
                             (list 'seh 'seh 'seh 'seh)
                             (list 'seh 'seh 'seh 'raed)
                             (list 'seh 'seh 'seh 'seh)
                             (list 'seh 'raed 'seh 'seh)
                             (list 'seh 'raed 'seh 'raed)
                             (list 'seh 'raed 'seh 'seh)
                             (list 'seh 'raed 'raed 'seh)
                             (list 'seh 'raed 'raed 'raed)
                             (list 'seh 'raed 'raed 'seh)
                             (list 'seh 'raed 'seh 'seh)
                             (list 'seh 'raed 'seh 'raed)
                             (list 'seh 'raed 'seh 'seh)
                             (list 'seh 'seh 'seh 'seh)
                             (list 'seh 'seh 'seh 'raed)
                             (list 'seh 'seh 'seh 'seh)
                             (list 'seh 'seh 'raed 'seh)
                             (list 'seh 'seh 'raed 'raed)
                             (list 'seh 'seh 'raed 'seh)
                             (list 'seh 'seh 'seh 'seh)
                             (list 'seh 'seh 'seh 'raed)
                             (list 'seh 'seh 'seh 'seh)))))

(suite ordered-prepend-to-each-source ()
  (test empty-source
    (ordered-prepend-to-each-source 'eto (finite-source))
    1 (source?)
    (pipe finite-source->list empty-list?))
  (test list-with-elements
    (ordered-prepend-to-each-source
      'uidte
      (finite-source (list 'msbtvs 'trra 'eth)
                     (list 'dafeaei 'auen 'csli)
                     (list 'pnthsun 'coatue 'sasna)
                     (list 'wihonyr 'reotost 'aeet)))
    1 (source?)
    (pipe finite-source->list
          (match? (list-of= (list-of= symbol=?))
                  (list (list 'uidte 'msbtvs 'trra 'eth)
                        (list 'uidte 'dafeaei 'auen 'csli)
                        (list 'uidte 'pnthsun 'coatue 'sasna)
                        (list 'uidte 'wihonyr 'reotost 'aeet))))))

(suite ordered-prepend-each-to-each-source ()
  (test left-empty
    (ordered-prepend-each-to-each-source
      (list)
      (finite-source (list 'stn 'twaa)
                     (list 'weoseob 'sid)
                     (list 'hhf 'imt)
                     (list 'fhhnst 'etambf)))
    1 (source?)
    (pipe finite-source->list empty-list?))
  (test right-empty
    (ordered-prepend-each-to-each-source
      (list 'ault 'wuslrs 'niehda 'habers 'ackdia)
      (finite-source))
    1 (source?)
    (pipe finite-source->list empty-list?))
  (test non-trivial
    (ordered-prepend-each-to-each-source
      (list 'ogl 'eniio 'shlitoa)
      (finite-source (list 'ede 'ntapte 'rsloaea 'sita)
                     (list 'qotu 'cmitsyo 'tge 'dibiie)
                     (list 'elieiya 'mau 'agtdant 'osorsen)))
    1 (source?)
    (pipe finite-source->list
          (match? (list-of= (list-of= symbol=?))
                  (list (list 'ogl 'ede 'ntapte 'rsloaea 'sita)
                        (list 'ogl 'qotu 'cmitsyo 'tge 'dibiie)
                        (list 'ogl 'elieiya 'mau 'agtdant 'osorsen)
                        (list 'eniio 'ede 'ntapte 'rsloaea 'sita)
                        (list 'eniio 'qotu 'cmitsyo 'tge 'dibiie)
                        (list 'eniio 'elieiya 'mau 'agtdant 'osorsen)
                        (list 'shlitoa 'ede 'ntapte 'rsloaea 'sita)
                        (list 'shlitoa 'qotu 'cmitsyo 'tge 'dibiie)
                        (list 'shlitoa 'elieiya 'mau 'agtdant
                              'osorsen))))))

(suite ordered-Cartesian-power-source ()
  (test minimal-case
    (ordered-Cartesian-power-source (list) 0)
    1 (source?)
    (pipe finite-source->list
          (match? (list-of= list=?) (list (list)))))
  (test zero-exponent
    (ordered-Cartesian-power-source (list 'ntufev 'hhrsyrh 'rere 'xdnoedi)
                                    0)
    1 (source?)
    (pipe finite-source->list
          (match? (list-of= list=?) (list (list)))))
  (test empty-list
    (ordered-Cartesian-power-source (list) 419)
    1 (source?)
    (pipe finite-source->list empty-list?))
  (test typical-case
    (ordered-Cartesian-power-source (list 392 535 808) 3)
    1 (source?)
    (pipe finite-source->list
          (match? (list-of= (list-of= =)) (list (list 392 392 392)
                                                (list 392 392 535)
                                                (list 392 392 808)
                                                (list 392 535 392)
                                                (list 392 535 535)
                                                (list 392 535 808)
                                                (list 392 808 392)
                                                (list 392 808 535)
                                                (list 392 808 808)
                                                (list 535 392 392)
                                                (list 535 392 535)
                                                (list 535 392 808)
                                                (list 535 535 392)
                                                (list 535 535 535)
                                                (list 535 535 808)
                                                (list 535 808 392)
                                                (list 535 808 535)
                                                (list 535 808 808)
                                                (list 808 392 392)
                                                (list 808 392 535)
                                                (list 808 392 808)
                                                (list 808 535 392)
                                                (list 808 535 535)
                                                (list 808 535 808)
                                                (list 808 808 392)
                                                (list 808 808 535)
                                                (list 808 808 808))))))

(suite position-in
    ((natural-number-or-failure? (^vel natural-number? not))) 
  (test empty-list
    (position-in 'eele (list))
    1 (natural-number-or-failure?)
    not)
  (test minimal-success
    (position-in 613 (list 613))
    1 (natural-number-or-failure?)
    (^et natural-number? zero?))
  (test find-at-front
    (position-in 73 (list 73 378 916 89 366))
    1 (natural-number-or-failure?)
    (^et natural-number? zero?))
  (test find-at-end
    (position-in 843 (list 310 951 524 279 602 677 96 61 71 440 933 159 797
                           70 948 372 711 410 149 439 374 435 462 955 933
                           809 549 282 132 449 400 631 727 164 615 843))
    1 (natural-number-or-failure?)
    (^et natural-number? (match? = 35)))
  (test find-in-middle
    (position-in 627 (list 54 640 921 167 575 728 461 547 706 310 627 479
                           277 289 808 576))
    1 (natural-number-or-failure?)
    (^et natural-number? (match? = 10)))
  (test long-failure
    (position-in 'hyh (list 'osaoe 'luteni 'geatra 'hfdse 'mtdu 'muv 'xivit
                         'ogw 'aews 'dpehhn 'aonbif 'fpeslhl))
    1 (natural-number-or-failure?)
    not))

(suite Cartesian-product-ranker ()
  (test minimal
    (Cartesian-product-ranker (list 'enehtwc) (list 'eecom))
    1 (procedure?)
    (lambda (ranker)
      (suite minimal-internal ()
        (test only
          (ranker (cons 'enehtwc 'eecom))
          1 (natural-number?)
          zero?))))
  (test typical
    (Cartesian-product-ranker (list 'tydya 'one 'addesg 'nottvhn)
                              (list 'sfy 'haoosit 'oaaofu))
    1 (procedure?)
    (lambda (ranker)
      (suite typical-internal ()
        (test zero-rank
          (ranker (cons 'tydya 'sfy))
          1 (natural-number?)
          zero?)
        (test last-rank
          (ranker (cons 'nottvhn 'oaaofu))
          1 (natural-number?)
          (match? = 11))
        (test mid-rank
          (ranker (cons 'addesg 'haoosit))
          1 (natural-number?)
          (match? = 7))))))

(suite Cartesian-product-unranker
    ((symbol-pair? (pair-of symbol? symbol?))
     (symbol-pair=? (pair-of= symbol=? symbol=?)))
  (test minimal
    (Cartesian-product-unranker (list 'cdsr) (list 'ses))
    1 (procedure?)
    (lambda (unranker)
      (suite minimal-internal ()
        (test only
          (unranker 0)
          1 (symbol-pair?)
          (match? symbol-pair=? (cons 'cdsr 'ses))))))
  (test typical
    (Cartesian-product-unranker
      (list 'see 'ooadsai 'edothig 'epe 'seetlsi 'rhn)
      (list 'aoeuiw 'eilsha 'erfbs 'stdul 'dpet 'swck 'eeooeta 'sthts
            'trmso 'tssrht 'enifroe))
    1 (procedure?)
    (lambda (unranker)
      (suite typical-internal ()
        (test zero-rank
          (unranker 0)
          1 (symbol-pair?)
          (match? symbol-pair=? (cons 'see 'aoeuiw)))
        (test last-rank
          (unranker 65)
          1 (symbol-pair?)
          (match? symbol-pair=? (cons 'rhn 'enifroe)))
        (test mid-rank
          (unranker 26)
          1 (symbol-pair?)
          (match? symbol-pair=? (cons 'edothig 'dpet)))))))

(suite positional-weights ((natlist? (list-of natural-number?))
                           (natlist=? (list-of= =)))
  (test no-bases
    (positional-weights (list))
    1 (natlist?)
    (match? natlist=? (list 1)))
  (test single-base
    (positional-weights (list 912))
    1 (natlist?)
    (match? natlist=? (list 912 1)))
  (test variable-bases
    (positional-weights (list 8 5 3 4 9 2 7))
    1 (natlist?)
    (match? natlist=? (list 60480 7560 1512 504 126 14 7 1))))

(suite generalized-Cartesian-product-ranker ()
  (test minimal
    (generalized-Cartesian-product-ranker (list 'ueleou))
    1 (procedure?)
    (lambda (ranker)
      (suite minimal-internal ()
        (test only
          (ranker (list 'ueleou))
          1 (natural-number?)
          zero?))))
  (test single-list
    (generalized-Cartesian-product-ranker (list 'ueeh 'iedoato 'csstt 'gdc
                                                'ooas 'hpltl 'fntit))
    1 (procedure?)
    (lambda (ranker)
      (suite single-list-internal ()
        (test zero-rank
          (ranker (list 'ueeh))
          1 (natural-number?)
          zero?)
        (test last-rank
          (ranker (list 'fntit))
          1 (natural-number?)
          (match? = 6))
        (test mid-rank
          (ranker (list 'ooas))
          1 (natural-number?)
          (match? = 4)))))
  (test multi-list
    (generalized-Cartesian-product-ranker
      (list 'otm 'heea 'aeltau 'harenp 'elogloe)
      (list 'wag 'hryac 'hbn)
      (list 'ttrni 'caitktg 'eein 'mleasdo 'twttdf 'gec)
      (list 'iewa 'eja 'urauw 'irmr 'ohr 'eikra 'oamavqi 'otn 'fpaega))
    1 (procedure?)
    (lambda (ranker)
      (suite multi-list-internal ()
        (test zero-rank
          (ranker (list 'otm 'wag 'ttrni 'iewa))
          1 (natural-number?)
          zero?)
        (test last-rank
          (ranker (list 'elogloe 'hbn 'gec 'fpaega))
          1 (natural-number?)
          (match? = 809))
        (test mid-rank
          (ranker (list 'aeltau 'hryac 'mleasdo 'ohr))
          1 (natural-number?)
          (match? = 409))))))

(suite separate-indices ((natlist? (list-of natural-number?))
                         (natlist=? (list-of= =)))
  (test minimal
    (separate-indices 0 (list 1))
    1 (natlist?)
    (match? natlist=? (list 0)))
  (test zero
    (separate-indices 0 (list 1296 216 72 12 3 1))
    1 (natlist?)
    (match? natlist=? (list 0 0 0 0 0 0)))
  (test typical
    (separate-indices 801 (list 1296 216 72 12 3 1))
    1 (natlist?)
    (match? natlist=? (list 0 3 2 0 3 0))))

(suite generalized-Cartesian-product-unranker
    ((symlist? (list-of symbol?))
     (symlist=? (list-of= symbol=?)))
  (test minimal
    (generalized-Cartesian-product-unranker (list 'raensdt))
    1 (procedure?)
    (lambda (unranker)
      (suite minimal-internal ()
        (test only
          (unranker 0)
          1 (symlist?)
          (match? symlist=? (list 'raensdt))))))
  (test single-list
    (generalized-Cartesian-product-unranker
      (list 'dtapas 'kde 'twsusa 'naeeo 'bdiso))
    1 (procedure?)
    (lambda (unranker)
      (suite single-list-internal ()
        (test zero-rank
          (unranker 0)
          1 (symlist?)
          (match? symlist=? (list 'dtapas)))
        (test last-rank
          (unranker 4)
          1 (symlist?)
          (match? symlist=? (list 'bdiso)))
        (test mid-rank
          (unranker 2)
          1 (symlist?)
          (match? symlist=? (list 'twsusa))))))
  (test multi-list
    (generalized-Cartesian-product-unranker
      (list 'eimo 'hcf 'tmatoe 'natul 'ids)
      (list 'etignit 'boh 'cwtrt 'piinitp)
      (list 'essnsy 'nhlen 'iasls 'whes 'eiespe 'hoouald)
      (list 'arnetmm 'bllit 'cya)
      (list 'hdeuta 'ahid 'ern 'dve 'retseio 'iie 'sstdsos 'fdepalb
            'egmeuos))
    1 (procedure?)
    (lambda (unranker)
      (suite multi-list-internal ()
        (test zero-rank
          (unranker 0)
          1 (symlist?)
          (match? symlist=?
                  (list 'eimo 'etignit 'essnsy 'arnetmm 'hdeuta)))
        (test last-rank
          (unranker 3239)
          1 (symlist?)
          (match? symlist=? (list 'ids 'piinitp 'hoouald 'cya 'egmeuos)))
        (test mid-rank
          (unranker 2476)
          1 (symlist?)
          (match? symlist=? (list 'natul 'piinitp 'nhlen 'cya 'ahid)))))))

;;; Test the procedures from the section on list selections.

(suite suffixes ((right-type? (bag-of (list-of symbol?)))
                 (equiv? (bag-of= (list-of= symbol=?))))
  (test empty-list
    (suffixes (list))
    1 (right-type?)
    (match? equiv? (bag (list))))
  (test typical-case
    (suffixes (list 'ohse 'ptmbhod 'nrig 'oad))
    1 (right-type?)
    (match? equiv? (bag (list 'ohse 'ptmbhod 'nrig 'oad)
                        (list 'ptmbhod 'nrig 'oad)
                        (list 'nrig 'oad)
                        (list 'oad)
                        (list)))))

(suite ordered-non-empty-suffixes
    ((right-type? (list-of (list-of symbol?))) 
     (equiv? (list-of= (list-of= symbol=?))))
  (test empty-list
    (ordered-non-empty-suffixes (list))
    1 (right-type?)
    empty-list?)
  (test singleton
    (ordered-non-empty-suffixes (list 'nah))
    1 (right-type?)
    (match? equiv? (list (list 'nah))))
  (test typical
    (ordered-non-empty-suffixes (list 'wan 'avveruu 'oenens 'lwilao
                                      'acpote 'aita))
    1 (right-type?)
    (match? equiv?
            (list (list 'wan 'avveruu 'oenens 'lwilao 'acpote 'aita)
                  (list 'avveruu 'oenens 'lwilao 'acpote 'aita)
                  (list 'oenens 'lwilao 'acpote 'aita)
                  (list 'lwilao 'acpote 'aita)
                  (list 'acpote 'aita)
                  (list 'aita)))))                         

(suite prefixes ((right-type? (bag-of (list-of symbol?)))
                 (equiv? (bag-of= (list-of= symbol=?))))
  (test empty-list
    (prefixes (list))
    1 (right-type?)
    (match? equiv? (bag (list))))
  (test typical-case
    (prefixes (list 'pmcdlb 'efwoi 'nbel 'aaeltl 'iearrei))
    1 (right-type?)
    (match? equiv? (bag (list)
                        (list 'pmcdlb)
                        (list 'pmcdlb 'efwoi)
                        (list 'pmcdlb 'efwoi 'nbel)
                        (list 'pmcdlb 'efwoi 'nbel 'aaeltl)
                        (list 'pmcdlb 'efwoi 'nbel 'aaeltl 'iearrei)))))

(suite alternative-sublists ((right-type? (bag-of (list-of symbol?)))
                             (equiv? (bag-of= (list-of= symbol=?))))
  (test empty-list
    (alternative-sublists (list))
    1 (right-type?)
    (match? equiv? (bag (list))))
  (test singleton-list
    (alternative-sublists (list 'aceralr))
    1 (right-type?)
    (match? equiv? (bag (list) (list 'aceralr) (list))))
  (test longer-list
    (alternative-sublists (list 'hohsa 'floulhw 'ewngiat 'hdlotn 'eiuhr))
    1 (right-type?)
    (match? equiv? (bag (list)
                        (list 'hohsa)
                        (list 'hohsa 'floulhw)
                        (list 'hohsa 'floulhw 'ewngiat)
                        (list 'hohsa 'floulhw 'ewngiat 'hdlotn)
                        (list 'hohsa 'floulhw 'ewngiat 'hdlotn 'eiuhr)
                        (list)
                        (list 'floulhw)
                        (list 'floulhw 'ewngiat)
                        (list 'floulhw 'ewngiat 'hdlotn)
                        (list 'floulhw 'ewngiat 'hdlotn 'eiuhr)
                        (list)
                        (list 'ewngiat)
                        (list 'ewngiat 'hdlotn)
                        (list 'ewngiat 'hdlotn 'eiuhr)
                        (list)
                        (list 'hdlotn)
                        (list 'hdlotn 'eiuhr)
                        (list)
                        (list 'eiuhr)
                        (list)))))

(suite non-empty-prefixes ((right-type? (bag-of (list-of symbol?)))
                           (equiv? (bag-of= (list-of= symbol=?))))
  (test empty-list
    (non-empty-prefixes (list))
    1 (right-type?)
    empty-bag?)
  (test singleton
    (non-empty-prefixes (list 'odstt))
    1 (right-type?)
    (match? equiv? (bag (list 'odstt))))
  (test typical-case
    (non-empty-prefixes (list 'nwhot 'spaea 'atyeit 'wsisd 'tttnae 'fco))
    1 (right-type?)
    (match? equiv?
            (bag (list 'nwhot)
                 (list 'nwhot 'spaea)
                 (list 'nwhot 'spaea 'atyeit)
                 (list 'nwhot 'spaea 'atyeit 'wsisd)
                 (list 'nwhot 'spaea 'atyeit 'wsisd 'tttnae)
                 (list 'nwhot 'spaea 'atyeit 'wsisd 'tttnae 'fco)))))

(suite sublists ((right-type? (bag-of (list-of symbol?)))
                 (equiv? (bag-of= (list-of= symbol=?))))
  (test empty-list
    (sublists (list))
    1 (right-type?)
    empty-bag?)
  (test singleton-list
    (sublists (list 'npiih))
    1 (right-type?)
    (match? equiv? (bag (list 'npiih))))
  (test longer-list
    (sublists (list 'hnosnsa 'tce 'nsod 'thnj))
    1 (right-type?)
    (match? equiv? (bag (list 'hnosnsa)
                        (list 'hnosnsa 'tce)
                        (list 'hnosnsa 'tce 'nsod)
                        (list 'hnosnsa 'tce 'nsod 'thnj)
                        (list 'tce)
                        (list 'tce 'nsod)
                        (list 'tce 'nsod 'thnj)
                        (list 'nsod)
                        (list 'nsod 'thnj)
                        (list 'thnj)))))

(suite termial ()
  (test zero
    (termial 0)
    1 (natural-number?)
    zero?)
  (test one
    (termial 1)
    1 (natural-number?)
    (match? = 1))
  (test typical
    (termial 191)
    1 (natural-number?)
    (match? = 18336)))

(suite sublist-ranker ()
  (test minimal
    (sublist-ranker (list 'eur))
    1 (procedure?)
    (lambda (ranker)
      (suite minimal-internal ()
        (test only
          (ranker (list 'eur))
          1 (natural-number?)
          zero?))))
  (test typical
    (sublist-ranker (list 'srfler 'wbtt 'tbrcan 'slthei 'ncon 'erahe))
    1 (procedure?)
    (lambda (ranker)
      (suite typical-internal ()
        (test zero-rank
          (ranker (list 'srfler))
          1 (natural-number?)
          zero?)
        (test mid-rank
          (ranker (list 'wbtt 'tbrcan 'slthei))
          1 (natural-number?)
          (match? = 8))
        (test last-rank
          (ranker (list 'erahe))
          1 (natural-number?)
          (match? = 20))))))          

(suite antitermial ()
  (test zero
    (antitermial 0)
    2 (natural-number? natural-number?)
    (match? = 0 0))
  (test one
    (antitermial 1)
    2 (natural-number? natural-number?)
    (match? = 1 0))
  (test two
    (antitermial 2)
    2 (natural-number? natural-number?)
    (match? = 1 1))
  (test three
    (antitermial 3)
    2 (natural-number? natural-number?)
    (match? = 2 0))
  (test typical
    (antitermial 217564)
    2 (natural-number? natural-number?)
    (match? = 659 94))
  (test just-short
    (antitermial 154289)
    2 (natural-number? natural-number?)
    (match? = 554 554))
  (test exact
    (antitermial 3486)
    2 (natural-number? natural-number?)
    (match? = 83 0)))

(suite sublist-unranker ()
  (test minimal
    (sublist-unranker (list 'nwawsp))
    1 (procedure?)
    (lambda (unranker)
      (suite minimal-internal ()
        (test only
          (unranker 0)
          1 ((list-of symbol?))
          (match? (list-of= symbol=?) (list 'nwawsp))))))
  (test typical
    (sublist-unranker (list 'kyris 'hsoyy 'ttcheu 'nohsde 'rranmee 'ntabe
                            'tsbhntf 'ceoif 'nnlats 'tte 'ddiew))
    1 (procedure?)
    (lambda (unranker)
      (suite typical-internal ((right-type? (list-of symbol?))
                               (equiv? (list-of= symbol=?)))
        (test zero-rank
          (unranker 0)
          1 (right-type?)
          (match? equiv? (list 'kyris)))
        (test last-rank
          (unranker 65)
          1 (right-type?)
          (match? equiv? (list 'ddiew)))
        (test mid-rank
          (unranker 36)
          1 (right-type?)
          (match? equiv? (list 'nohsde 'rranmee 'ntabe 'tsbhntf 'ceoif
                               'nnlats 'tte)))))))

(suite sections ()
  (test minimal
    (sections (list 'aoaws) 1)
    1 ((bag-of (list-of symbol?)))
    (match? (bag-of= (list-of= symbol=?)) (bag (list 'aoaws))))
  (test typical
    (sections (list 'alpha 'beta 'gamma 'delta 'epsilon) 3)
    1 ((bag-of (list-of symbol?)))
    (match? (bag-of= (list-of= symbol=?))
            (bag (list 'alpha 'beta 'gamma)
                 (list 'beta 'gamma 'delta)
                 (list 'gamma 'delta 'epsilon))))
  (test full-length
    (sections (list 'hmhy 'dedra 'tnuai 'petdew 'svrv 'yoniua 'clyog
                    'igurwo)
              8)
    1 ((bag-of (list-of symbol?)))
    (match? (bag-of= (list-of= symbol=?))
            (bag (list 'hmhy 'dedra 'tnuai 'petdew 'svrv 'yoniua 'clyog
                       'igurwo)))))

(suite subsequences ((right-type? (bag-of (list-of symbol?)))
                     (equiv? (bag-of= (list-of= symbol=?))))
  (test empty-list
    (subsequences (list))
    1 ((bag-of list?))
    (match? (bag-of= list=?) (bag (list))))
  (test singleton-list
    (subsequences (list 'swmf))
    1 (right-type?)
    (match? equiv? (bag (list) (list 'swmf))))
  (test longer-list
    (subsequences (list 'alpha 'beta 'gamma 'delta))
    1 (right-type?)
    (match? equiv? (bag (list)
                        (list 'delta)
                        (list 'gamma)
                        (list 'gamma 'delta)
                        (list 'beta)
                        (list 'beta 'delta)
                        (list 'beta 'gamma)
                        (list 'beta 'gamma 'delta)
                        (list 'alpha)
                        (list 'alpha 'delta)
                        (list 'alpha 'gamma)
                        (list 'alpha 'gamma 'delta)
                        (list 'alpha 'beta)
                        (list 'alpha 'beta 'delta)
                        (list 'alpha 'beta 'gamma)
                        (list 'alpha 'beta 'gamma 'delta))))
  (test duplicates
    (subsequences (list 'htt 'fcn 'htt 'fcn 'fcn))
    1 (right-type?)
    (match? equiv? (bag (list)
                        (list 'fcn)
                        (list 'fcn)
                        (list 'fcn 'fcn)
                        (list 'htt)
                        (list 'htt 'fcn)
                        (list 'htt 'fcn)
                        (list 'htt 'fcn 'fcn)
                        (list 'fcn)
                        (list 'fcn 'fcn)
                        (list 'fcn 'fcn)
                        (list 'fcn 'fcn 'fcn)
                        (list 'fcn 'htt)
                        (list 'fcn 'htt 'fcn)
                        (list 'fcn 'htt 'fcn)
                        (list 'fcn 'htt 'fcn 'fcn)
                        (list 'htt)
                        (list 'htt 'fcn)
                        (list 'htt 'fcn)
                        (list 'htt 'fcn 'fcn)
                        (list 'htt 'htt)
                        (list 'htt 'htt 'fcn)
                        (list 'htt 'htt 'fcn)
                        (list 'htt 'htt 'fcn 'fcn)
                        (list 'htt 'fcn)
                        (list 'htt 'fcn 'fcn)
                        (list 'htt 'fcn 'fcn)
                        (list 'htt 'fcn 'fcn 'fcn)
                        (list 'htt 'fcn 'htt)
                        (list 'htt 'fcn 'htt 'fcn)
                        (list 'htt 'fcn 'htt 'fcn)
                        (list 'htt 'fcn 'htt 'fcn 'fcn)))))

(suite subsequence->list-of-Booleans ((bool-list? (list-of boolean?))
                                      (bool-list=? (list-of= boolean=?)))
  (test minimal
    (subsequence->list-of-Booleans (list))
    1 (procedure?)
    (lambda (mapper)
      (suite minimal-internal ()
        (test only
          (mapper (list))
          1 (bool-list?)
          empty-list?))))
  (test typical
    (subsequence->list-of-Booleans (list 'alpha 'beta 'gamma 'delta))
    1 (procedure?)
    (lambda (mapper)
      (suite typical-internal ()
        (test none
          (mapper (list))
          1 (bool-list?)
          (match? bool-list=? (list #f #f #f #f)))
        (test all
          (mapper (list 'alpha 'beta 'gamma 'delta))
          1 (bool-list?)
          (match? bool-list=? (list #t #t #t #t)))
        (test some
          (mapper (list 'alpha 'gamma))
          1 (bool-list?)
          (match? bool-list=? (list #t #f #t #f)))))))

(suite list-of-Booleans->subsequence ()
  (test minimal
    (list-of-Booleans->subsequence (list))
    1 (procedure?)
    (lambda (mapper)
      (suite minimal-internal ()
        (test only
          (mapper (list))
          1 (list?)
          empty-list?))))
  (test typical
    (list-of-Booleans->subsequence (list 'sebai 'rltsots 'raya 'uih 'aeer))
    1 (procedure?)
    (lambda (mapper)
      (suite typical-internal ()
        (test none
          (mapper (list #f #f #f #f #f))
          1 ((list-of symbol?))
          empty-list?)
        (test all
          (mapper (list #t #t #t #t #t))
          1 ((list-of symbol?))
          (match? (list-of= symbol=?)
                  (list 'sebai 'rltsots 'raya 'uih 'aeer)))
        (test some
          (mapper (list #t #f #f #t #f))
          1 ((list-of symbol?))
          (match? (list-of= symbol=?) (list 'sebai 'uih)))))))

(suite alternative-subsequences ((right-type? (bag-of (list-of symbol?)))
                                 (equiv? (bag-of= (list-of= symbol=?))))
  (test empty-list
    (alternative-subsequences (list))
    1 ((bag-of list?))
    (match? (bag-of= list=?) (bag (list))))
  (test singleton-list
    (alternative-subsequences (list 'cnl))
    1 (right-type?)
    (match? equiv? (bag (list) (list 'cnl))))
  (test longer-list
    (alternative-subsequences (list  'iuh 'anu 'yytel 'itw))
    1 (right-type?)
    (match? equiv? (bag (list)
                        (list 'itw)
                        (list 'yytel)
                        (list 'yytel 'itw)
                        (list 'anu)
                        (list 'anu 'itw)
                        (list 'anu 'yytel)
                        (list 'anu 'yytel 'itw)
                        (list 'iuh)
                        (list 'iuh 'itw)
                        (list 'iuh 'yytel)
                        (list 'iuh 'yytel 'itw)
                        (list 'iuh 'anu)
                        (list 'iuh 'anu 'itw)
                        (list 'iuh 'anu 'yytel)
                        (list 'iuh 'anu 'yytel 'itw))))
  (test duplicates
    (alternative-subsequences (list 'iie 'ata 'enkod 'iie 'ata))
    1 (right-type?)
    (match? equiv? (bag (list)
                        (list 'ata)
                        (list 'iie)
                        (list 'iie 'ata)
                        (list 'enkod)
                        (list 'enkod 'ata)
                        (list 'enkod 'iie)
                        (list 'enkod 'iie 'ata)
                        (list 'ata)
                        (list 'ata 'ata)
                        (list 'ata 'iie)
                        (list 'ata 'iie 'ata)
                        (list 'ata 'enkod)
                        (list 'ata 'enkod 'ata)
                        (list 'ata 'enkod 'iie)
                        (list 'ata 'enkod 'iie 'ata)
                        (list 'iie)
                        (list 'iie 'ata)
                        (list 'iie 'iie)
                        (list 'iie 'iie 'ata)
                        (list 'iie 'enkod)
                        (list 'iie 'enkod 'ata)
                        (list 'iie 'enkod 'iie)
                        (list 'iie 'enkod 'iie 'ata)
                        (list 'iie 'ata)
                        (list 'iie 'ata 'ata)
                        (list 'iie 'ata 'iie)
                        (list 'iie 'ata 'iie 'ata)
                        (list 'iie 'ata 'enkod)
                        (list 'iie 'ata 'enkod 'ata)
                        (list 'iie 'ata 'enkod 'iie)
                        (list 'iie 'ata 'enkod 'iie 'ata)))))

(suite selections ((right-type? (bag-of (list-of symbol?)))
                   (equiv? (bag-of= (list-of= symbol=?))))
  (test empty-list
    (selections (list) 0)
    1 ((bag-of list?))
    (match? (bag-of= list=?) (bag (list))))
  (test empty-selections
    (selections (list 'aru 'rtedte 'tuo 'omgk 'orr) 0)
    1 ((bag-of list?))
    (match? (bag-of= list=?) (bag (list))))
  (test whole-list
    (selections (list 'shlcrsu 'frloii 'cwtheet 'ivnrol 'ains 'enfaety
                      'eth)
                7)
    1 (right-type?)
    (match? equiv? (bag (list 'shlcrsu 'frloii 'cwtheet 'ivnrol 'ains
                              'enfaety 'eth))))
  (test typical-case
    (selections (list 'ehtin 'her 'crlsd 'sshtdt 'emhjlg 'nhnems 'lrc) 3)
    1 (right-type?)
    (match? equiv? (bag (list 'ehtin 'her 'crlsd)
                        (list 'ehtin 'her 'sshtdt)
                        (list 'ehtin 'her 'emhjlg)
                        (list 'ehtin 'her 'nhnems)
                        (list 'ehtin 'her 'lrc)
                        (list 'ehtin 'crlsd 'sshtdt)
                        (list 'ehtin 'crlsd 'emhjlg)
                        (list 'ehtin 'crlsd 'nhnems)
                        (list 'ehtin 'crlsd 'lrc)
                        (list 'ehtin 'sshtdt 'emhjlg)
                        (list 'ehtin 'sshtdt 'nhnems)
                        (list 'ehtin 'sshtdt 'lrc)
                        (list 'ehtin 'emhjlg 'nhnems)
                        (list 'ehtin 'emhjlg 'lrc)
                        (list 'ehtin 'nhnems 'lrc)
                        (list 'her 'crlsd 'sshtdt)
                        (list 'her 'crlsd 'emhjlg)
                        (list 'her 'crlsd 'nhnems)
                        (list 'her 'crlsd 'lrc)
                        (list 'her 'sshtdt 'emhjlg)
                        (list 'her 'sshtdt 'nhnems)
                        (list 'her 'sshtdt 'lrc)
                        (list 'her 'emhjlg 'nhnems)
                        (list 'her 'emhjlg 'lrc)
                        (list 'her 'nhnems 'lrc)
                        (list 'crlsd 'sshtdt 'emhjlg)
                        (list 'crlsd 'sshtdt 'nhnems)
                        (list 'crlsd 'sshtdt 'lrc)
                        (list 'crlsd 'emhjlg 'nhnems)
                        (list 'crlsd 'emhjlg 'lrc)
                        (list 'crlsd 'nhnems 'lrc)
                        (list 'sshtdt 'emhjlg 'nhnems)
                        (list 'sshtdt 'emhjlg 'lrc)
                        (list 'sshtdt 'nhnems 'lrc)
                        (list 'emhjlg 'nhnems 'lrc)))))

(suite binomial ()
  (test minimal
    (binomial 0 0)
    1 (natural-number?)
    (match? = 1))
  (test choose-none
    (binomial 361 0)
    1 (natural-number?)
    (match? = 1))
  (test choose-all
    (binomial 678 678)
    1 (natural-number?)
    (match? = 1))
  (test typical
    (binomial 363 141)
    1 (natural-number?)
    (match? = 88839685527476801574928546518382063070434966915834066577965827264996049896300422341700746081698718022640)))

(suite selection-rank ()
  (test base-case
    (selection-rank (list) 0 (list))
    1 (natural-number?)
    zero?)
  (test minimal-selection
    (selection-rank (list 'amm 'aah 'tnhnwte 'tsdyhle 'nde) 0 (list))
    1 (natural-number?)
    zero?)
  (test maximal-selection
    (selection-rank (list 'utisbas 'htteeee 'nsido 'gehit 'tedidf 'ugn)
                    6
                    (list 'utisbas 'htteeee 'nsido 'gehit 'tedidf 'ugn))
    1 (natural-number?)
    zero?)
  (test zero-rank
    (selection-rank (list 'ohr 'tcdig 'idra 'etaheei 'giei 'ussohh
                          'feoensn)
                    4
                    (list 'ohr 'tcdig 'idra 'etaheei))
    1 (natural-number?)
    zero?)
  (test last-rank
    (selection-rank (list 'seusuy 'hns 'towo 'hvwra 'oirfee 'itotlis)
                    3
                    (list 'hvwra 'oirfee 'itotlis))
    1 (natural-number?)
    (match? = 19))
  (test mid-rank
    (selection-rank (list 'fddtn 'diei 'cstrg 'donod 'oioan 'thumohi
                          'ywabut)
                    4
                    (list 'fddtn 'cstrg 'oioan 'ywabut))
    1 (natural-number?)
    (match? = 14)))

(suite selection-unrank ((symlist? (list-of symbol?))
                         (symlist=? (list-of= symbol=?)))
  (test base-case
    (selection-unrank (list) 0 0)
    1 (list?)
    empty-list?)
  (test minimal-selection
    (selection-unrank (list 'uos 'nmrota 'osaitlp 'uota 'yam 'elnm 'mhk)
                      0
                      0)
     1 (list?)
     empty-list?)
  (test maximal-selection
    (selection-unrank (list 'one 'otef 'osee 'ayi) 4 0)
    1 (symlist?)
    (match? symlist=? (list 'one 'otef 'osee 'ayi)))
  (test zero-rank
    (selection-unrank (list 'dtrs 'oboomg 'sigot 'aess 'nsl 'vulpea 'ebe
                            'thncut)
                      5
                      0)
    1 (symlist?)
    (match? symlist=? (list 'dtrs 'oboomg 'sigot 'aess 'nsl)))
  (test last-rank
    (selection-unrank (list 'roa 'aea 'tnht 'ayoihyr 'oara 'nth 'pah)
                      4
                      34)
    1 (symlist?)
    (match? symlist=? (list 'ayoihyr 'oara 'nth 'pah)))
  (test midrank
    (selection-unrank (list 'wudnor 'ewdddwd 'thh 'utiaera 'ktoq 'antsaoo
                            'iia)
                      3
                      12)
    1 (symlist?)
    (match? symlist=? (list 'wudnor 'ktoq 'antsaoo))))

;;; Test the procedures from the section on bag selections.

(suite put-into-each-bag ((right-type? (bag-of (bag-of symbol?)))
                          (equiv? (bag-of= (bag-of= symbol=?))))
  (test empty-bag
    (put-into-each-bag 'amnam (bag))
    1 (bag?)
    empty-bag?)
  (test singleton-bag
    (put-into-each-bag 'nei (bag (bag 'satctkr 'iutotdc 'tklcflh 'tttol)))
    1 (right-type?)
    (match? equiv? (bag (bag 'nei 'satctkr 'iutotdc 'tklcflh 'tttol))))
  (test more-members
    (put-into-each-bag 'aqlmelr (bag (bag 'cromt 'ihees 'detlhki 'sodannv)
                                     (bag 'saatn)
                                     (bag)
                                     (bag 'sstbb 'lrf 'thenc 'lhga 'dovdled
                                          'ngpld 'ougoidv 'orrm 'tts)))
    1 (right-type?)
    (match? equiv? (bag (bag 'aqlmelr 'cromt 'ihees 'detlhki 'sodannv)
                        (bag 'aqlmelr 'saatn)
                        (bag 'aqlmelr)
                        (bag 'aqlmelr 'sstbb 'lrf 'thenc 'lhga 'dovdled
                             'ngpld 'ougoidv 'orrm 'tts)))))

(suite subbags ((right-type? (bag-of (bag-of symbol?)))
                (equiv? (bag-of= (bag-of= symbol=?))))
  (test empty-bag
    (subbags (bag))
    1 (right-type?)
    (match? equiv? (bag (bag))))
  (test singleton-bag
    (subbags (bag 'swmf))
    1 (right-type?)
    (match? equiv? (bag (bag) (bag 'swmf))))
  (test larger-bag
    (subbags (bag 'hemr 'hynst 'unsnh 'etro 'nni))
    1 (right-type?)
    (match? equiv? (bag (bag)
                        (bag 'nni)
                        (bag 'etro)
                        (bag 'etro 'nni)
                        (bag 'unsnh)
                        (bag 'unsnh 'nni)
                        (bag 'unsnh 'etro)
                        (bag 'unsnh 'etro 'nni)
                        (bag 'hynst)
                        (bag 'hynst 'nni)
                        (bag 'hynst 'etro)
                        (bag 'hynst 'etro 'nni)
                        (bag 'hynst 'unsnh)
                        (bag 'hynst 'unsnh 'nni)
                        (bag 'hynst 'unsnh 'etro)
                        (bag 'hynst 'unsnh 'etro 'nni)
                        (bag 'hemr)
                        (bag 'hemr 'nni)
                        (bag 'hemr 'etro)
                        (bag 'hemr 'etro 'nni)
                        (bag 'hemr 'unsnh)
                        (bag 'hemr 'unsnh 'nni)
                        (bag 'hemr 'unsnh 'etro)
                        (bag 'hemr 'unsnh 'etro 'nni)
                        (bag 'hemr 'hynst)
                        (bag 'hemr 'hynst 'nni)
                        (bag 'hemr 'hynst 'etro)
                        (bag 'hemr 'hynst 'etro 'nni)
                        (bag 'hemr 'hynst 'unsnh)
                        (bag 'hemr 'hynst 'unsnh 'nni)
                        (bag 'hemr 'hynst 'unsnh 'etro)
                        (bag 'hemr 'hynst 'unsnh 'etro 'nni))))
  (test duplicates
    (subbags (bag 'neuohg 'fra 'neuohg 'fra 'fra))
    1 (right-type?)
    (match? equiv? (bag (bag)
                        (bag 'fra)
                        (bag 'fra)
                        (bag 'fra 'fra)
                        (bag 'neuohg)
                        (bag 'neuohg 'fra)
                        (bag 'neuohg 'fra)
                        (bag 'neuohg 'fra 'fra)
                        (bag 'fra)
                        (bag 'fra 'fra)
                        (bag 'fra 'fra)
                        (bag 'fra 'fra 'fra)
                        (bag 'fra 'neuohg)
                        (bag 'fra 'neuohg 'fra)
                        (bag 'fra 'neuohg 'fra)
                        (bag 'fra 'neuohg 'fra 'fra)
                        (bag 'neuohg)
                        (bag 'neuohg 'fra)
                        (bag 'neuohg 'fra)
                        (bag 'neuohg 'fra 'fra)
                        (bag 'neuohg 'neuohg)
                        (bag 'neuohg 'neuohg 'fra)
                        (bag 'neuohg 'neuohg 'fra)
                        (bag 'neuohg 'neuohg 'fra 'fra)
                        (bag 'neuohg 'fra)
                        (bag 'neuohg 'fra 'fra)
                        (bag 'neuohg 'fra 'fra)
                        (bag 'neuohg 'fra 'fra 'fra)
                        (bag 'neuohg 'fra 'neuohg)
                        (bag 'neuohg 'fra 'neuohg 'fra)
                        (bag 'neuohg 'fra 'neuohg 'fra)
                        (bag 'neuohg 'fra 'neuohg 'fra 'fra))))) 

(suite combinations ((right-type? (bag-of (bag-of symbol?)))
                     (equiv? (bag-of= (bag-of= symbol=?))))
  (test empty-bag
    (combinations (bag) 0)
    1 (right-type?)
    (match? equiv? (bag (bag))))
  (test empty-combination
    (combinations (bag 'tsree 'badokfm 'dhymwer 'ntes 'tmeoo 'srdlcel) 0)
    1 (right-type?)
    (match? equiv? (bag (bag))))
  (test whole-bag-combinations
    (combinations (bag 'insds 'nthde 'ane 'tsaws 'fpaog 'egnsh 'gsaoen 'ing
                       'hnaoeih)
                  9)
    1 (right-type?)
    (match? equiv? (bag (bag 'insds 'nthde 'ane 'tsaws 'fpaog 'egnsh
                             'gsaoen 'ing 'hnaoeih))))
  (test typical-case
    (combinations (bag 'oeor 'hrety 'rlkw 'ononhav 'anil 'laueth 'ehant) 3)
    1 (right-type?)
    (match? equiv? (bag (bag 'oeor 'hrety 'rlkw)
                        (bag 'oeor 'hrety 'ononhav)
                        (bag 'oeor 'hrety 'anil)
                        (bag 'oeor 'hrety 'laueth)
                        (bag 'oeor 'hrety 'ehant)
                        (bag 'oeor 'rlkw 'ononhav)
                        (bag 'oeor 'rlkw 'anil)
                        (bag 'oeor 'rlkw 'laueth)
                        (bag 'oeor 'rlkw 'ehant)
                        (bag 'oeor 'ononhav 'anil)
                        (bag 'oeor 'ononhav 'laueth)
                        (bag 'oeor 'ononhav 'ehant)
                        (bag 'oeor 'anil 'laueth)
                        (bag 'oeor 'anil 'ehant)
                        (bag 'oeor 'laueth 'ehant)
                        (bag 'hrety 'rlkw 'ononhav)
                        (bag 'hrety 'rlkw 'anil)
                        (bag 'hrety 'rlkw 'laueth)
                        (bag 'hrety 'rlkw 'ehant)
                        (bag 'hrety 'ononhav 'anil)
                        (bag 'hrety 'ononhav 'laueth)
                        (bag 'hrety 'ononhav 'ehant)
                        (bag 'hrety 'anil 'laueth)
                        (bag 'hrety 'anil 'ehant)
                        (bag 'hrety 'laueth 'ehant)
                        (bag 'rlkw 'ononhav 'anil)
                        (bag 'rlkw 'ononhav 'laueth)
                        (bag 'rlkw 'ononhav 'ehant)
                        (bag 'rlkw 'anil 'laueth)
                        (bag 'rlkw 'anil 'ehant)
                        (bag 'rlkw 'laueth 'ehant)
                        (bag 'ononhav 'anil 'laueth)
                        (bag 'ononhav 'anil 'ehant)
                        (bag 'ononhav 'laueth 'ehant)
                        (bag 'anil 'laueth 'ehant)))))

(suite combinations-source ((equiv? (bag-of= (bag-of= symbol=?))))
  (test empty-bag
    (combinations-source (bag) 0)
    1 (source?)
    (run finite-source->list
         (fold-list bag put-into-bag)
         (match? equiv? (bag (bag)))))
  (test empty-combination
    (combinations-source (bag 'ddreniv 'adonwle 'isfo 'eid) 0)
    1 (source?)
    (run finite-source->list
         (fold-list bag put-into-bag)
         (match? equiv? (bag (bag)))))
  (test whole-bag-combinations
    (combinations-source (bag 'oasr 'hhf 'tho 'onls 'peynd 'ibtw 'mteoos)
                         7)
    1 (source?)
    (run finite-source->list
         (fold-list bag put-into-bag)
         (match? equiv?
                 (bag (bag 'oasr 'hhf 'tho 'onls 'peynd 'ibtw 'mteoos)))))
  (test typical-case
    (combinations-source (bag 'varryh 'iannn 'hfmmrs 'uorhne 'wlie 'ion
                              'eocut)
                         4)
    1 (source?)
    (run finite-source->list
         (fold-list bag put-into-bag)
         (match? equiv? (bag (bag 'varryh 'iannn 'hfmmrs 'uorhne)
                             (bag 'varryh 'iannn 'hfmmrs 'wlie)
                             (bag 'varryh 'iannn 'hfmmrs 'ion)
                             (bag 'varryh 'iannn 'hfmmrs 'eocut)
                             (bag 'varryh 'iannn 'uorhne 'wlie)
                             (bag 'varryh 'iannn 'uorhne 'ion)
                             (bag 'varryh 'iannn 'uorhne 'eocut)
                             (bag 'varryh 'iannn 'wlie 'ion)
                             (bag 'varryh 'iannn 'wlie 'eocut)
                             (bag 'varryh 'iannn 'ion 'eocut)
                             (bag 'varryh 'hfmmrs 'uorhne 'wlie)
                             (bag 'varryh 'hfmmrs 'uorhne 'ion)
                             (bag 'varryh 'hfmmrs 'uorhne 'eocut)
                             (bag 'varryh 'hfmmrs 'wlie 'ion)
                             (bag 'varryh 'hfmmrs 'wlie 'eocut)
                             (bag 'varryh 'hfmmrs 'ion 'eocut)
                             (bag 'varryh 'uorhne 'wlie 'ion)
                             (bag 'varryh 'uorhne 'wlie 'eocut)
                             (bag 'varryh 'uorhne 'ion 'eocut)
                             (bag 'varryh 'wlie 'ion 'eocut)
                             (bag 'iannn 'hfmmrs 'uorhne 'wlie)
                             (bag 'iannn 'hfmmrs 'uorhne 'ion)
                             (bag 'iannn 'hfmmrs 'uorhne 'eocut)
                             (bag 'iannn 'hfmmrs 'wlie 'ion)
                             (bag 'iannn 'hfmmrs 'wlie 'eocut)
                             (bag 'iannn 'hfmmrs 'ion 'eocut)
                             (bag 'iannn 'uorhne 'wlie 'ion)
                             (bag 'iannn 'uorhne 'wlie 'eocut)
                             (bag 'iannn 'uorhne 'ion 'eocut)
                             (bag 'iannn 'wlie 'ion 'eocut)
                             (bag 'hfmmrs 'uorhne 'wlie 'ion)
                             (bag 'hfmmrs 'uorhne 'wlie 'eocut)
                             (bag 'hfmmrs 'uorhne 'ion 'eocut)
                             (bag 'hfmmrs 'wlie 'ion 'eocut)
                             (bag 'uorhne 'wlie 'ion 'eocut))))))

;;; Test the procedures from the section on permutations.

(suite permutations ((right-type? (bag-of (list-of symbol?)))
                     (equiv? (bag-of= (list-of= symbol=?))))
  (test empty-bag
    (permutations (bag))
    1 (right-type?)
    (match? equiv? (bag (list))))
  (test singleton-bag
    (permutations (bag 'pnv))
    1 (right-type?)
    (match? equiv? (bag (list 'pnv))))
  (test doubleton-bag
    (permutations (bag 'teitie 'nochuw))
    1 (right-type?)
    (match? equiv? (bag (list 'teitie 'nochuw)
                        (list 'nochuw 'teitie))))
  (test larger-bag
    (permutations (bag 'rac 'tsd 'uesemio 'nrsd))
    1 (right-type?)
    (match? equiv? (bag (list 'rac 'tsd 'uesemio 'nrsd)
                        (list 'rac 'tsd 'nrsd 'uesemio)
                        (list 'rac 'uesemio 'tsd 'nrsd)
                        (list 'rac 'uesemio 'nrsd 'tsd)
                        (list 'rac 'nrsd 'tsd 'uesemio)
                        (list 'rac 'nrsd 'uesemio 'tsd)
                        (list 'tsd 'rac 'uesemio 'nrsd)
                        (list 'tsd 'rac 'nrsd 'uesemio)
                        (list 'tsd 'uesemio 'rac 'nrsd)
                        (list 'tsd 'uesemio 'nrsd 'rac)
                        (list 'tsd 'nrsd 'rac 'uesemio)
                        (list 'tsd 'nrsd 'uesemio 'rac)
                        (list 'uesemio 'rac 'tsd 'nrsd)
                        (list 'uesemio 'rac 'nrsd 'tsd)
                        (list 'uesemio 'tsd 'rac 'nrsd)
                        (list 'uesemio 'tsd 'nrsd 'rac)
                        (list 'uesemio 'nrsd 'rac 'tsd)
                        (list 'uesemio 'nrsd 'tsd 'rac)
                        (list 'nrsd 'rac 'tsd 'uesemio)
                        (list 'nrsd 'rac 'uesemio 'tsd)
                        (list 'nrsd 'tsd 'rac 'uesemio)
                        (list 'nrsd 'tsd 'uesemio 'rac)
                        (list 'nrsd 'uesemio 'rac 'tsd)
                        (list 'nrsd 'uesemio 'tsd 'rac))))
  (test duplicates
    (permutations (bag 'acf 'lhnia 'acf))
    1 (right-type?)
    (match? equiv? (bag (list 'acf 'lhnia 'acf)
                        (list 'acf 'acf 'lhnia)
                        (list 'lhnia 'acf 'acf)
                        (list 'lhnia 'acf 'acf)
                        (list 'acf 'lhnia 'acf)
                        (list 'acf 'acf 'lhnia)))))

(suite all-but-position ((symlist? (list-of symbol?))
                         (symlist=? (list-of= symbol=?)))
  (test minimal
    (all-but-position (list 'aohl) 0)
    1 (symlist?)
    empty-list?)
  (test front
    (all-but-position (list 'ayd 'wuaodol 'eseydr 'nas 'onv) 0)
    1 (symlist?)
    (match? symlist=? (list 'wuaodol 'eseydr 'nas 'onv)))
  (test rear
    (all-but-position (list 'honh 'drs 'apabwro 'aolh 'sds 'hifao) 5)
    1 (symlist?)
    (match? symlist=? (list 'honh 'drs 'apabwro 'aolh 'sds)))
  (test middle
    (all-but-position (list 'aseesy 'ehh 'yuriii 'thwhwh 'aireks 'iieliee
                            'ritilfn)
                      3)
    1 (symlist?)
    (match? symlist=? (list 'aseesy 'ehh 'yuriii 'aireks 'iieliee
                            'ritilfn))))

(suite ordered-permutations ((right-type? (list-of (list-of symbol?)))
                             (equiv? (list-of= (list-of= symbol=?))))
  (test empty-list
    (ordered-permutations (list))
    1 (right-type?)
    (match? equiv? (list (list))))
  (test singleton-list
    (ordered-permutations (list 'nede))
    1 (right-type?)
    (match? equiv? (list (list 'nede))))
  (test doubleton-list
    (ordered-permutations (list 'frusho 'seoae))
    1 (right-type?)
    (match? equiv? (list (list 'frusho 'seoae)
                         (list 'seoae 'frusho))))
  (test larger-list
    (ordered-permutations (list 'tfbd 'rrteih 'osear 'dedaoa))
    1 (right-type?)
    (match? equiv? (list (list 'tfbd 'rrteih 'osear 'dedaoa)
                         (list 'tfbd 'rrteih 'dedaoa 'osear)
                         (list 'tfbd 'osear 'rrteih 'dedaoa)
                         (list 'tfbd 'osear 'dedaoa 'rrteih)
                         (list 'tfbd 'dedaoa 'rrteih 'osear)
                         (list 'tfbd 'dedaoa 'osear 'rrteih)
                         (list 'rrteih 'tfbd 'osear 'dedaoa)
                         (list 'rrteih 'tfbd 'dedaoa 'osear)
                         (list 'rrteih 'osear 'tfbd 'dedaoa)
                         (list 'rrteih 'osear 'dedaoa 'tfbd)
                         (list 'rrteih 'dedaoa 'tfbd 'osear)
                         (list 'rrteih 'dedaoa 'osear 'tfbd)
                         (list 'osear 'tfbd 'rrteih 'dedaoa)
                         (list 'osear 'tfbd 'dedaoa 'rrteih)
                         (list 'osear 'rrteih 'tfbd 'dedaoa)
                         (list 'osear 'rrteih 'dedaoa 'tfbd)
                         (list 'osear 'dedaoa 'tfbd 'rrteih)
                         (list 'osear 'dedaoa 'rrteih 'tfbd)
                         (list 'dedaoa 'tfbd 'rrteih 'osear)
                         (list 'dedaoa 'tfbd 'osear 'rrteih)
                         (list 'dedaoa 'rrteih 'tfbd 'osear)
                         (list 'dedaoa 'rrteih 'osear 'tfbd)
                         (list 'dedaoa 'osear 'tfbd 'rrteih)
                         (list 'dedaoa 'osear 'rrteih 'tfbd))))
  (test duplicates
    (ordered-permutations (list 'eahfohl 'mmrysnt 'iettrs 'mmrysnt))
    1 (right-type?)
    (match? equiv? (list (list 'eahfohl 'mmrysnt 'iettrs 'mmrysnt)
                         (list 'eahfohl 'mmrysnt 'mmrysnt 'iettrs)
                         (list 'eahfohl 'iettrs 'mmrysnt 'mmrysnt)
                         (list 'eahfohl 'iettrs 'mmrysnt 'mmrysnt)
                         (list 'eahfohl 'mmrysnt 'mmrysnt 'iettrs)
                         (list 'eahfohl 'mmrysnt 'iettrs 'mmrysnt)
                         (list 'mmrysnt 'eahfohl 'iettrs 'mmrysnt)
                         (list 'mmrysnt 'eahfohl 'mmrysnt 'iettrs)
                         (list 'mmrysnt 'iettrs 'eahfohl 'mmrysnt)
                         (list 'mmrysnt 'iettrs 'mmrysnt 'eahfohl)
                         (list 'mmrysnt 'mmrysnt 'eahfohl 'iettrs)
                         (list 'mmrysnt 'mmrysnt 'iettrs 'eahfohl)
                         (list 'iettrs 'eahfohl 'mmrysnt 'mmrysnt)
                         (list 'iettrs 'eahfohl 'mmrysnt 'mmrysnt)
                         (list 'iettrs 'mmrysnt 'eahfohl 'mmrysnt)
                         (list 'iettrs 'mmrysnt 'mmrysnt 'eahfohl)
                         (list 'iettrs 'mmrysnt 'eahfohl 'mmrysnt)
                         (list 'iettrs 'mmrysnt 'mmrysnt 'eahfohl)
                         (list 'mmrysnt 'eahfohl 'mmrysnt 'iettrs)
                         (list 'mmrysnt 'eahfohl 'iettrs 'mmrysnt)
                         (list 'mmrysnt 'mmrysnt 'eahfohl 'iettrs)
                         (list 'mmrysnt 'mmrysnt 'iettrs 'eahfohl)
                         (list 'mmrysnt 'iettrs 'eahfohl 'mmrysnt)
                         (list 'mmrysnt 'iettrs 'mmrysnt 'eahfohl)))))

(suite ordered-permutations-source
    ((equiv? (list-of= (list-of= symbol=?))))
  (test empty-list
    (ordered-permutations-source (list))
    1 (source?)
    (pipe finite-source->list (match? equiv? (list (list)))))
  (test singleton-list
    (ordered-permutations-source (list 'tndei))
    1 (source?)
    (pipe finite-source->list (match? equiv? (list (list 'tndei)))))
  (test doubleton-list
    (ordered-permutations-source (list 'tep 'ryferi))
    1 (source?)
    (pipe finite-source->list
          (match? equiv? (list (list 'tep 'ryferi)
                               (list 'ryferi 'tep)))))
  (test larger-list
    (ordered-permutations-source (list 'raisih 'yueur 'ntr))
    1 (source?)
    (pipe finite-source->list
          (match? equiv? (list (list 'raisih 'yueur 'ntr)
                               (list 'raisih 'ntr 'yueur)
                               (list 'yueur 'raisih 'ntr)
                               (list 'yueur 'ntr 'raisih)
                               (list 'ntr 'raisih 'yueur)
                               (list 'ntr 'yueur 'raisih)))))
  (test duplicates
    (ordered-permutations-source (list 'lsv 'egfiome 'looflo 'lsv))
    1 (source?)
    (pipe finite-source->list
          (match? equiv? (list (list 'lsv 'egfiome 'looflo 'lsv)
                               (list 'lsv 'egfiome 'lsv 'looflo)
                               (list 'lsv 'looflo 'egfiome 'lsv)
                               (list 'lsv 'looflo 'lsv 'egfiome)
                               (list 'lsv 'lsv 'egfiome 'looflo)
                               (list 'lsv 'lsv 'looflo 'egfiome)
                               (list 'egfiome 'lsv 'looflo 'lsv)
                               (list 'egfiome 'lsv 'lsv 'looflo)
                               (list 'egfiome 'looflo 'lsv 'lsv)
                               (list 'egfiome 'looflo 'lsv 'lsv)
                               (list 'egfiome 'lsv 'lsv 'looflo)
                               (list 'egfiome 'lsv 'looflo 'lsv)
                               (list 'looflo 'lsv 'egfiome 'lsv)
                               (list 'looflo 'lsv 'lsv 'egfiome)
                               (list 'looflo 'egfiome 'lsv 'lsv)
                               (list 'looflo 'egfiome 'lsv 'lsv)
                               (list 'looflo 'lsv 'lsv 'egfiome)
                               (list 'looflo 'lsv 'egfiome 'lsv)
                               (list 'lsv 'lsv 'egfiome 'looflo)
                               (list 'lsv 'lsv 'looflo 'egfiome)
                               (list 'lsv 'egfiome 'lsv 'looflo)
                               (list 'lsv 'egfiome 'looflo 'lsv)
                               (list 'lsv 'looflo 'lsv 'egfiome)
                               (list 'lsv 'looflo 'egfiome 'lsv))))))

(suite permutation-rank ()
  (test empty-permutation
    (permutation-rank (list) (list))
    1 (natural-number?)
    zero?)
  (test zero-rank
    (permutation-rank (list 'aheeeg 'eralht 'weltes 'onnh 'rboes 'vwrhaft)
                      (list 'aheeeg 'eralht 'weltes 'onnh 'rboes 'vwrhaft))
    1 (natural-number?)
    zero?)
  (test last-rank
    (permutation-rank
      (list 'adnltah 'ojha 'ciw 'des 'sto 'brttoph 'ettmeo 'mnwnl)
      (list 'mnwnl 'ettmeo 'brttoph 'sto 'des 'ciw 'ojha 'adnltah))
    1 (natural-number?)
    (match? = 40319))
  (test mid-rank
    (permutation-rank
      (list 'acwhio 'lnhtndt 'nhh 'bnr 'mhdcfo 'wcfoei 'aaeitsn)
      (list 'lnhtndt 'bnr 'aaeitsn 'acwhio 'nhh 'wcfoei 'mhdcfo))
    1 (natural-number?)
    (match? = 1057)))

(suite permutation-unrank ((symlist? (list-of symbol?))
                           (symlist=? (list-of= symbol=?)))
  (test empty-permutation
    (permutation-unrank (list) 0)
    1 (list?)
    empty-list?)
  (test zero-rank
    (permutation-unrank (list 'rgupu 'anne 'yasee 'rdyar 'oamr 'rip) 0)
    1 (symlist?)
    (match? symlist=? (list 'rgupu 'anne 'yasee 'rdyar 'oamr 'rip)))
  (test last-rank
    (permutation-unrank
      (list 'iow 'lds 'irote 'hfgmwe 'rpeanea 'etfh 'munror)
      5039)
    1 (symlist?)
    (match? symlist=?
            (list 'munror 'etfh 'rpeanea 'hfgmwe 'irote 'lds 'iow)))
  (test mid-rank
    (permutation-unrank (list 'nhlin 'ymno 'aees 'bhsatim 'ohtc 'hhimaps
                              'mec 'tieg 'hmmo)
                        102193)
    1 (symlist?)
    (match? symlist=? (list 'aees 'hhimaps 'ymno 'hmmo 'mec 'nhlin 'bhsatim
                            'tieg 'ohtc))))

;;; Test procedures from the section on partitions.

(suite remove-bag-from-bag ((right-type? (bag-of (bag-of symbol?)))
                            (equiv? (bag-of= (bag-of= symbol=?))))
  (test only-bag
    (remove-bag-from-bag (bag 'ehdaoa 'euwh 'dyat)
                         (bag (bag 'ehdaoa 'euwh 'dyat)))
    1 (right-type?)
    empty-bag?)
  (test larger-bag
    (remove-bag-from-bag (bag 'diul 'ong 'neeeots 'hssnnw)
                         (bag (bag 'eona 'nrlfgge 'moige 'iscuth)
                              (bag 'roh 'ennn)
                              (bag 'cotauot 'iat 'srvcn 'tooce 'hiatu)
                              (bag 'neeeots 'diul 'ong 'hssnnw)
                              (bag 'eteeei 'aaih 'shs)))
    1 (right-type?)
    (match? equiv? (bag (bag 'eona 'nrlfgge 'moige 'iscuth)
                        (bag 'roh 'ennn)
                        (bag 'cotauot 'iat 'srvcn 'tooce 'hiatu)
                        (bag 'eteeei 'aaih 'shs)))))

(suite extend-partition ((right-type? (bag-of (bag-of (bag-of symbol?))))
                         (equiv? (bag-of= (bag-of= (bag-of= symbol=?)))))
  (test null-partition
    (extend-partition 'neuoe (bag))
    1 (right-type?)
    (match? equiv? (bag (bag (bag 'neuoe)))))
  (test partition-of-singleton
    (extend-partition 'tixoe (bag (bag 'een)))
    1 (right-type?)
    (match? equiv? (bag (bag (bag 'een 'tixoe))
                        (bag (bag 'een) (bag 'tixoe)))))
  (test larger-partition
    (extend-partition 'hph (bag (bag 'rnre 'nrfvo)
                                (bag 'stoo 'fydtn)
                                (bag 'rou)))
    1 (right-type?)
    (match? equiv? (bag (bag (bag 'rnre 'nrfvo)
                             (bag 'stoo 'fydtn)
                             (bag 'rou)
                             (bag 'hph))
                        (bag (bag 'rnre 'nrfvo 'hph)
                             (bag 'stoo 'fydtn)
                             (bag 'rou))
                        (bag (bag 'rnre 'nrfvo)
                             (bag 'stoo 'fydtn 'hph)
                             (bag 'rou))
                        (bag (bag 'rnre 'nrfvo)
                             (bag 'stoo 'fydtn)
                             (bag 'rou 'hph))))))

(suite partitions ((right-type? (bag-of (bag-of (bag-of symbol?))))
                   (equiv? (bag-of= (bag-of= (bag-of= symbol=?)))))
  (test empty-bag
    (partitions (bag))
    1 ((bag-of bag?))
    (match? (bag-of= bag=?) (bag (bag))))
  (test singleton-bag
    (partitions (bag 'gipler))
    1 (right-type?)
    (match? equiv? (bag (bag (bag 'gipler)))))
  (test larger-bag
    (partitions (bag 'swwon 'osneh 'gaei 'phh))
    1 (right-type?)
    (match? equiv? (bag (bag (bag 'swwon 'osneh 'gaei 'phh))
                        (bag (bag 'swwon 'osneh 'gaei)
                             (bag 'phh))
                        (bag (bag 'swwon 'osneh 'phh)
                             (bag 'gaei))
                        (bag (bag 'swwon 'gaei 'phh)
                             (bag 'osneh))
                        (bag (bag 'osneh 'gaei 'phh)
                             (bag 'swwon))
                        (bag (bag 'swwon 'osneh)
                             (bag 'gaei 'phh))
                        (bag (bag 'swwon 'gaei)
                             (bag 'osneh 'phh))
                        (bag (bag 'swwon 'phh)
                             (bag 'osneh 'gaei))
                        (bag (bag 'swwon 'osneh)
                             (bag 'gaei)
                             (bag 'phh))
                        (bag (bag 'swwon 'gaei)
                             (bag 'osneh)
                             (bag 'phh))
                        (bag (bag 'swwon 'phh)
                             (bag 'osneh)
                             (bag 'gaei))
                        (bag (bag 'osneh 'gaei)
                             (bag 'swwon)
                             (bag 'phh))
                        (bag (bag 'osneh 'phh)
                             (bag 'swwon)
                             (bag 'gaei))
                        (bag (bag 'gaei 'phh)
                             (bag 'swwon)
                             (bag 'osneh))
                        (bag (bag 'swwon)
                             (bag 'osneh)
                             (bag 'gaei)
                             (bag 'phh))))))

(suite bounded-number-partitions
    ((right-type? (bag-of (bag-of natural-number?)))
     (equiv? (bag-of= (bag-of= =))))
  (test minimal
    (bounded-number-partitions 0 0)
    1 (right-type?)
    (match? equiv? (bag (bag))))
  (test zero-number
    (bounded-number-partitions 0 601)
    1 (right-type?)
    (match? equiv? (bag (bag))))
  (test zero-bound
    (bounded-number-partitions 707 0)
    1 (right-type?)
    empty-bag?)
  (test unit-bound
    (bounded-number-partitions 5 1)
    1 (right-type?)
    (match? equiv? (bag (bag 1 1 1 1 1))))
  (test bound-2
    (bounded-number-partitions 9 2)
    1 (right-type?)
    (match? equiv? (bag (bag 1 1 1 1 1 1 1 1 1)
                        (bag 2 1 1 1 1 1 1 1)
                        (bag 2 2 1 1 1 1 1)
                        (bag 2 2 2 1 1 1)
                        (bag 2 2 2 2 1))))
  (test higher-bound
    (bounded-number-partitions 9 4)
    1 (right-type?)
    (match? equiv? (bag (bag 1 1 1 1 1 1 1 1 1)
                        (bag 2 1 1 1 1 1 1 1)
                        (bag 2 2 1 1 1 1 1)
                        (bag 2 2 2 1 1 1)
                        (bag 2 2 2 2 1)
                        (bag 3 1 1 1 1 1 1)
                        (bag 3 2 1 1 1 1)
                        (bag 3 2 2 1 1)
                        (bag 3 2 2 2)
                        (bag 3 3 1 1 1)
                        (bag 3 3 2 1)
                        (bag 3 3 3)
                        (bag 4 1 1 1 1 1)
                        (bag 4 2 1 1 1)
                        (bag 4 2 2 1)
                        (bag 4 3 1 1)
                        (bag 4 3 2)
                        (bag 4 4 1)))))                        

(suite number-partitions ((right-type? (bag-of (bag-of natural-number?)))
                          (equiv? (bag-of= (bag-of= =))))

  (test zero
    (number-partitions 0)
    1 (right-type?)
    (match? equiv? (bag (bag))))
  (test one
    (number-partitions 1)
    1 (right-type?)
    (match? equiv? (bag (bag 1))))
  (test two
    (number-partitions 2)
    1 (right-type?)
    (match? equiv? (bag (bag 1 1) (bag 2))))
  (test typical
    (number-partitions 11)
    1 (right-type?)
    (match? equiv? (bag (bag 1 1 1 1 1 1 1 1 1 1 1)
                        (bag 2 1 1 1 1 1 1 1 1 1)
                        (bag 2 2 1 1 1 1 1 1 1)
                        (bag 2 2 2 1 1 1 1 1)
                        (bag 2 2 2 2 1 1 1)
                        (bag 2 2 2 2 2 1)
                        (bag 3 1 1 1 1 1 1 1 1)
                        (bag 3 2 1 1 1 1 1 1)
                        (bag 3 2 2 1 1 1 1)
                        (bag 3 2 2 2 1 1)
                        (bag 3 2 2 2 2)
                        (bag 3 3 1 1 1 1 1)
                        (bag 3 3 2 1 1 1)
                        (bag 3 3 2 2 1)
                        (bag 3 3 3 1 1)
                        (bag 3 3 3 2)
                        (bag 4 1 1 1 1 1 1 1)
                        (bag 4 2 1 1 1 1 1)
                        (bag 4 2 2 1 1 1)
                        (bag 4 2 2 2 1)
                        (bag 4 3 1 1 1 1)
                        (bag 4 3 2 1 1)
                        (bag 4 3 2 2)
                        (bag 4 3 3 1)
                        (bag 4 4 1 1 1)
                        (bag 4 4 2 1)
                        (bag 4 4 3)
                        (bag 5 1 1 1 1 1 1)
                        (bag 5 2 1 1 1 1)
                        (bag 5 2 2 1 1)
                        (bag 5 2 2 2)
                        (bag 5 3 1 1 1)
                        (bag 5 3 2 1)
                        (bag 5 3 3)
                        (bag 5 4 1 1)
                        (bag 5 4 2)
                        (bag 5 5 1)
                        (bag 6 1 1 1 1 1)
                        (bag 6 2 1 1 1)
                        (bag 6 2 2 1)
                        (bag 6 3 1 1)
                        (bag 6 3 2)
                        (bag 6 4 1)
                        (bag 6 5)
                        (bag 7 1 1 1 1)
                        (bag 7 2 1 1)
                        (bag 7 2 2)
                        (bag 7 3 1)
                        (bag 7 4)
                        (bag 8 1 1 1)
                        (bag 8 2 1)
                        (bag 8 3)
                        (bag 9 1 1)
                        (bag 9 2)
                        (bag 10 1)
                        (bag 11)))))                        

;;; copyright (C) 2011, 2017 John David Stone

;;; This program is free software.  You may redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation -- either version 3 of the License, or (at
;;; your option) any later version.  A copy of the GNU General Public
;;; License is available on the World Wide Web at
;;;
;;;                http://www.gnu.org/licenses/gpl.html
;;;
;;; This program is distributed in the hope that it will be useful, but
;;; WITHOUT ANY WARRANTY -- without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;;; General Public License for more details.
