;;; This is the test program for the testing procedures themselves.

;;; John David Stone
;;; Department of Computer Science
;;; Grinnell College

;;; created March 9, 1999
;;; last revised January 12, 2017

(import (scheme base)
        (support testing)
        (support testing-procedures))

(suite exact-rational? ()
  (test non-number
    (exact-rational? 'teaerc)
    1 (boolean?)
    not)
  (test non-integer-number
    (exact-rational? 233/856)
    1 (boolean?)
    (lambda (result)
      (boolean=? result #t)))
  (test inexact-number
    (exact-rational? 368.898)
    1 (boolean?)
    not)
  (test integer
    (exact-rational? 976)
    1 (boolean?)
    (lambda (result)
      (boolean=? result #t))))

(suite true? ()
  (test non-boolean
    (true? 'iun)
    1 (boolean?)
    not)
  (test false
    (true? #f)
    1 (boolean?)
    not)
  (test true
    (true? #t)
    1 (boolean?)
    (lambda (b) b)))   ;; Non-circular enough for you?

(suite match? ()
  (test one-number
    (match? = 499)
    1 (procedure?)
    (lambda (matcher)
      (suite one-number-internal ()
        (test yes
          (matcher 499)
          1 (boolean?)
          (lambda (result)
            (boolean=? result #t)))
        (test no
          (matcher 211)
          1 (boolean?)
          not))))
  (test one-string
    (match? string=? "rhlyyit")
    1 (procedure?)
    (lambda (matcher)
      (suite one-string-internal ()
        (test yes
          (matcher "rhlyyit")
          1 (boolean?)
          (lambda (result)
            (boolean=? result #t)))
        (test no
          (matcher "cionps")
          1 (boolean?)
          not))))
  (test one-symbol
    (match? symbol=? 'hawt)
    1 (procedure?)
    (lambda (matcher)
      (suite one-symbol-internal ()
        (test yes
          (matcher 'hawt)
          1 (boolean?)
          (lambda (result)
            (boolean=? result #t)))
        (test no
          (matcher 'bmheg)
          1 (boolean?)
          not))))
  (test several-values
    (match? = 314 225 512 162 280 166)
    1 (procedure?)
    (lambda (matcher)
      (suite several-values-internal ()
        (test yes
          (matcher 314 225 512 162 280 166)
          1 (boolean?)
          (lambda (result)
            (boolean=? result #t)))
        (test partial-match
          (matcher 314 225 512 255 280 166)
          1 (boolean?)
          not)
        (test no-match
          (matcher 651 433 253 395 43 802)
          1 (boolean?)
          not)))))

(suite source? ()
  (test yes
    (source? (letrec ((onwards (lambda () (values 592 onwards))))
               onwards))
    1 (boolean?)
    true?)
  (test no
    (source? 20/442)
    1 (boolean?)
    not))

;;; copyright (C) 2011, 2016 John David Stone

;;; This program is free software.  You may redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation -- either version 3 of the License, or (at
;;; your option) any later version.  A copy of the GNU General Public
;;; License is available on the World Wide Web at
;;;
;;;                http://www.gnu.org/licenses/gpl.html
;;;
;;; This program is distributed in the hope that it will be useful, but
;;; WITHOUT ANY WARRANTY -- without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;;; General Public License for more details.
