;;; This is the test program for the suite and test syntaxes.

;;; John David Stone
;;; Department of Computer Science
;;; Grinnell College

;;; created March 9, 1999
;;; last revised December 28, 2016

;;; Import the syntaxes to be tested.

(import (scheme base)
        (support testing))

(suite suite-and-test-syntaxes

  ;; Define the local procedure exact-number?.

  ((exact-number? (lambda (something)
                    (and (number? something) (exact? something)))))

  ;; A minimal case: a do-nothing expression that always passes a
  ;; do-nothing test.

  (test do-nothing
    (values)
    0 ()
    (lambda () #t))

  ;; Test whether zero is even.

  (test predicted-success
    (test should-succeed
      0
      1 (exact-number?)
      even?)
    1 (boolean?)
    (lambda (result)
      (boolean=? result #t)))

  ;; Test whether zero is odd.  The test syntax causes a failure
  ;; notice to be printed in this case -- don't be fooled into
  ;; thinking that the syntax is in error.

  (test predicted-failure
    (test should-fail
      0
      1 (exact-number?)
      odd?)
    1 (boolean?)
    not)

  ;; Make sure that the test syntax deals correctly with multiple
  ;; values.

  (test multiple-values
    (values 946 192 172 135 52)
    5 (exact-number? exact-number? exact-number? exact-number?
       exact-number?)
    >)

  ;; Make sure that the variant syntax in which the joint test is
  ;; omitted works correctly, too.

  (test variant-syntax
    (values #f (list))
    2 (boolean? null?)))

;;; An empty suite should report no errors.

(suite empty-suite ())

;;; copyright (C) 2011, 2016 John David Stone

;;; This program is free software.  You may redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation -- either version 3 of the License, or (at
;;; your option) any later version.  A copy of the GNU General Public
;;; License is available on the World Wide Web at
;;;
;;;                http://www.gnu.org/licenses/gpl.html
;;;
;;; This program is distributed in the hope that it will be useful, but
;;; WITHOUT ANY WARRANTY -- without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;;; General Public License for more details.
