;;; Testing the code from "The tool box"

;;; John David Stone
;;; Department of Computer Science
;;; Grinnell College

;;; created March 14, 1999
;;; last revised January 12, 2017

(import (afp primitives)
        (afp arithmetic)
        (afp extras)
        (afp constant-procedures)
        (afp procedure-sections)
        (afp couplers)
        (afp adapters)
        (afp recursion-managers)
        (afp predicate-operations)
        (afp natural-numbers)
        (support testing)
        (support testing-procedures)
        (only (scheme base) gcd))

;;; Test the procedures in the "List mapping" section.

(suite sum-of-squares ()
  (test no-arguments
    (sum-of-squares)
    1 (exact-integer?)
    zero?)
  (test one-argument
    (sum-of-squares 532)
    1 (exact-integer?)
    (match? = (square 532)))
  (test several-arguments
    (sum-of-squares 284 938 900 191 173 312)
    1 (exact-integer?)
    (match? = (+ (square 284) (square 938) (square 900) (square 191)
                 (square 173) (square 312)))))

(suite sum-of-cubes ()
  (test no-arguments
    (sum-of-cubes)
    1 (exact-integer?)
    zero?)
  (test one-argument
    (sum-of-cubes 317)
    1 (exact-integer?)
    (match? = (* 317 317 317)))
  (test several-arguments
    (sum-of-cubes 841 314 792 322 629 439)
    1 (exact-integer?)
    (match? = (+ (* 841 841 841) (* 314 314 314) (* 792 792 792)
                 (* 322 322 322) (* 629 629 629) (* 439 439 439))))

  ;; Same tests, but for the alternative definition.

  (test alternative-no-arguments
    (alternative-sum-of-cubes)
    1 (exact-integer?)
    zero?)
  (test alternative-one-argument
    (alternative-sum-of-cubes 317)
    1 (exact-integer?)
    (match? = (* 317 317 317)))
  (test alternative-several-arguments
    (alternative-sum-of-cubes 841 314 792 322 629 439)
    1 (exact-integer?)
    (match? = (+ (* 841 841 841) (* 314 314 314) (* 792 792 792)
                 (* 322 322 322) (* 629 629 629) (* 439 439 439)))))

;;; Test the procedures in the "Constant procedures" section.

(suite constant-procedures ()
  (test values?-nullary
    (values?)
    1 (boolean?)
    true?)
  (test values?-multary
    (values? 'hauaaio 'pwte 'soh 'igidare 'ntetteg 'ior 'faapey)
    1 (boolean?)
    true?)
  (test ignorer
    (constant 'uesc)
    1 (procedure?)
    (lambda (ignorer)
      (suite constant-ignorer ()
        (test ignorer-nullary
         (ignorer)
         1 (symbol?)
         (match? symbol=? 'uesc))
       (test ignorer-multary
         (ignorer 'uant 'coideh 'tert 'wfrars 'hslhe)
         1 (symbol?)
         (match? symbol=? 'uesc)))))
  (test multiple-valued-ignorer
    (constant 'rhtu 'enisls 'tuget)
    1 (procedure?)
    (lambda (multiple-valued-ignorer)
      (suite constant-multiple-valued-ignorer ()
        (test simple
          (multiple-valued-ignorer 'hbapd 'rire 'sutnhne 'osre)
          3 (symbol? symbol? symbol?)
          (match? symbol=? 'rhtu 'enisls 'tuget)))))
  (test create-nullary
    (create)
    1 (procedure?)
    (lambda (creator)
      (suite create-nullary-internal ()
        (test run-it
          (creator)
          0 ()))))
  (test create-unary
    (create 'tcor)
    1 (procedure?)
    (lambda (creator)
      (suite create-unary-internal ()
        (test create-it
          (creator)
          1 (symbol?)
          (match? symbol=? 'tcor)))))
  (test create-multary
    (create 'ahtgrs 'lds 'vth 'aatiohf 'slaoo 'ier)
    1 (procedure?)
    (lambda (creator)
      (suite create-multary-internal ()
        (test create-them
          (creator)
          6 (symbol? symbol? symbol? symbol? symbol? symbol?)
          (match? symbol=? 'ahtgrs 'lds 'vth 'aatiohf 'slaoo 'ier)))))
  (test black-hole-nullary
    (black-hole)
    0 ())
  (test black-hole-multary
    (black-hole 'uealhgn 'atqm 'cmsitd 'opg 'ldledr 'rrahlhc 'eafe
                'edoaha 'tnne 'stndn 'phtny 'oevtcw)
    0 ())
  (test length
    (lambda (ls)
      (apply + (map (constant 1) ls)))
    1 (procedure?)
    (lambda (length)
      (suite length-internal ()
        (test null-length
          (length (list))
          1 (exact-integer?)
          zero?)
        (test multi-length
          (length (list 'efis 'olhum 'iny 'ois))
          1 (exact-integer?)
          (match? = 4))))))

;;; Test the procedures in the "Procedure sections" section.

(suite power-of-two ()
  (test zero
    (power-of-two 0)
    1 (exact-integer?)
    (match? = 1))
  (test large-integer
    (power-of-two 963)
    1 (exact-integer?)
    (match? = (expt 2 963)))
  (test negative-integer
    (power-of-two -26)
    1 (real?)
    (match? = (/ (expt 2 26)))))

(suite alternative-square ((alternative-square (sect expt <> 2)))
  (test simple-square
    (alternative-square 32)
    1 (exact-integer?)
    (match? = 1024))
  (test negative-rational-square
    (alternative-square -22/7)
    1 (exact-rational?)
    (match? = 484/49))
  (test zero-square
    (alternative-square 0)
    1 (exact-integer?)
    zero?))

(suite invoke ()
  (test typical
    (invoke div-and-mod 443 137)
    2 (exact-integer? exact-integer?)
    (match? = 3 32))
  (test no-arguments
    (invoke (constant 315))
    1 (exact-integer?)
    (match? = 315))
  (test no-values
    (invoke black-hole 891 499 27)
    0 ()))

(suite alternative-invoke ()
  (test typical
    (alternative-invoke div-and-mod 887 592)
    2 (exact-integer? exact-integer?)
    (match? = 1 295))
  (test no-arguments
    (alternative-invoke (constant 483))
    1 (exact-integer?)
    (match? = 483))
  (test no-values
    (alternative-invoke black-hole 636 381)
    0 ()))

(suite add1 ()
  (test typical
    (add1 -319)
    1 (exact-integer?)
    (match? = -318))
  (test gigantic
    (add1 221822736823961656693477082320)
    1 (exact-integer?)
    (match? = 221822736823961656693477082321))
  (test non-integer
    (add1 -38/7)
    1 (exact-rational?)
    (match? = -31/7)))

(suite sub1 ()
  (test typical
    (sub1 -894)
    1 (exact-integer?)
    (match? = -895))
  (test gigantic
    (sub1 280778143272199525968541)
    1 (exact-integer?)
    (match? =  280778143272199525968540))
  (test non-integer
    (sub1 626/899)
    1 (exact-rational?)
    (match? = -273/899)))

(suite double ()
  (test zero
    (double 0)
    1 (exact-integer?)
    zero?)
  (test modest
    (double -993/28)
    1 (exact-rational?)
    (match? = -993/14))
  (test gigantic
    (double 29956477652842064210089158)
    1 (exact-integer?)
    (match? = 59912955305684128420178316)))

(suite curry ()

  ;; If curry is given a unary procedure, it returns a procedure that
  ;; constructs thunks.

  (test thunk-maker
    (curry square)
    1 (procedure?)
    (lambda (curried)
      (suite thunk-maker-internal ()
        (test square-thunk
          (curried 619)
          1 (procedure?)
          (lambda (thunk)
            (suite thunk-applier-internal ()
              (test thaw-thunk
                (thunk)
                1 (exact-integer?)
                (match? = (square 619)))))))))

  ;; If it is given a binary procedure, it returns a procedure that accepts
  ;; the first argument and returns a procedure that accepts the second
  ;; argument and carries out the computation.

  (test binary-procedure
    (curry expt)
    1 (procedure?)
    (lambda (curried)
      (suite curried-binary-internal ()
        (test curried-binary-procedure
          (curried 328)
          1 (procedure?)
          (lambda (to-the-power)
            (suite curried-binary-even-more-internal ()
              (test other-shoe-drops
                (to-the-power 541)
                1 (exact-integer?)
                (match? = (expt 328 541)))))))))

  ;; The curry procedure also works if it is given a procedure of higher
  ;; arity.  Only the first argument is split out.

  (test higher-arity
    (curry -)
    1 (procedure?)
    (lambda (curried)
      (suite curried-higher-arity-internal ()
        (test curried-higher-arity
          (curried 23)
          1 (procedure?)
          (lambda (from-twenty-three)
            (suite curried-higher-arity-even-more-internal ()
              (test many-shoes
                (from-twenty-three 1 8 19)
                1 (exact-integer?)
                (match? = (- 23 1 8 19))))))))))

(suite equal-to ()
  (test typical
    (equal-to 'hamclon)
    1 (procedure?)
    (lambda (equal-to-hamclon?)
      (suite equal-to-internal ()
        (test affirmative
          (equal-to-hamclon? 'hamclon)
          1 (boolean?)
          true?)
        (test nice-try
          (equal-to-hamclon? (list 'hamclon))
          1 (boolean?)
          not)
        (test negative
          (equal-to-hamclon? 'etroys)
          1 (boolean?)
          not)))))

;;; Test the procedures in the "Couplers" section.

(suite integer-quotient? ()
  (test simple
    (lambda (dividend divisor)
      (integer? (/ dividend divisor)))
    1 (procedure?)
    (lambda (integer-quotient?)
      (suite integer-quotient?-internal ()
        (test yes
          (integer-quotient? 930 15)
          1 (boolean?)
          true?)
        (test no
          (integer-quotient? 553 754)
          1 (boolean?)
          not)))))

(suite compose ()

  ;; In the minimal, do-nothing case, neither inner nor outer takes any
  ;; arguments or produces any values.  The resulting composer also takes
  ;; no arguments and produces no values.

  (test minimal
    (compose (lambda () (values)) (lambda () (values)))
    1 (procedure?)
    (lambda (result)
      (suite minimal-internal ()
        (test minimal-in-inaction
          (result)
          0 ()))))

  ;; The most straightforward case is the one in which each procedure takes
  ;; one argument and returns one result.  The composer takes one argument,
  ;; provides it to inner, collects the intermediate result, provides it to
  ;; outer, and returns the final result.

  (test single-values
    (compose square halve)
    1 (procedure?)
    (lambda (composer)
      (suite single-values-internal ()
        (test square-of-half
          (composer 871)
          1 (exact-integer?)
          (match? = (square (halve 871)))))))

  ;; When inner takes no arguments, the composer, taking no arguments,
  ;; applies outer to the fixed values produced by inner.

  (test nullary-inner
    (compose square (lambda () 465))
    1 (procedure?)
    (lambda (composer)
      (suite nullary-inner-internal ()
        (test square-of-constant
          (composer)
          1 (exact-integer?)
          (match? = (square 465))))))

  ;; When outer takes no arguments, inner produces none, and the
  ;; composer discards the arguments supplied and simply invokes
  ;; outer.

  (test no-intermediates
    (compose (lambda () 'mtoas)
             (lambda (unua dua tria) (values)))
    1 (procedure?)
    (lambda (composer)
      (suite no-intermediates-internal ()
        (test invoke-outer
          (composer 'edf 'tsmt 'dgascn)
          1 (symbol?)
          (match? symbol=? 'mtoas)))))

  ;; When inner returns multiple values, outer accepts them.

  (test multiple-intermediates
    (compose expt (lambda (num)
                    (values (square num) (halve num))))
    1 (procedure?)
    (lambda (composer)
      (suite multiple-intermediates-internal ()
        (test square-to-half-power
          (composer 752)
          1 (exact-integer?)
          (match? = (expt (square 752) (halve 752)))))))

  ;; When inner is a variable-arity procedure, so is the composer.

  (test variable-arity-inner
    (compose square +)
    1 (procedure?)
    (lambda (composer)
      (suite variable-arity-inner-internal ()
        (test add-and-square
          (composer 76 958 576 250 263)
          1 (exact-integer?)
          (match? = (square (+ 76 958 576 250 263)))))))

  ;; When outer is a variable-arity procedure, inner can return 
  ;; arbitrarily many values.

  (test variable-arity-outer
    (compose + (lambda arguments
                 (apply values (map halve arguments))))
    1 (procedure?)
    (lambda (composer)
      (suite variable-arity-outer-internal ()
        (test halve-and-add
          (composer 989 69 577 820 475 478)
          1 (exact-integer?)
          (match? = (+ (halve 989) (halve 69) (halve 577) (halve 820)
                       (halve 475) (halve 478))))))))

(suite pipe ()

  ;; In the minimal, do-nothing case, neither earlier nor later takes any
  ;; arguments or produces any values.  The resulting composite procedure
  ;; also takes no arguments and produces no values.

  (test minimal
    (pipe (lambda () (values)) (lambda () (values)))
    1 (procedure?)
    (lambda (result)
      (suite minimal-internal ()
        (test minimal-in-inaction
          (result)
          0 ()))))

  ;; The most straightforward case is the one in which each procedure takes
  ;; one argument and returns one result.  The piper takes one argument,
  ;; provides it to earlier, collects the intermediate result, provides it
  ;; to later, and returns the final result.

  (test single-values
    (pipe square halve)
    1 (procedure?)
    (lambda (piper)
      (suite single-values-internal ()
        (test square-of-half
          (piper 663)
          1 (exact-integer?)
          (match? = (halve (square 663)))))))

  ;; When earlier takes no arguments, the piper, taking no arguments,
  ;; applies later to the fixed values produced by earlier.

  (test nullary-earlier
    (pipe (lambda () 663) square)
    1 (procedure?)
    (lambda (piper)
      (suite nullary-earlier-internal ()
        (test square-of-constant
          (piper)
          1 (exact-integer?)
          (match? = (square 663))))))

  ;; When later takes no arguments, earlier produces none, and the piper
  ;; discards the arguments supplied and simply invoke later.

  (test no-intermediates
    (pipe (lambda (unua dua) (values)) (lambda () 'irtiida))
    1 (procedure?)
    (lambda (piper)
      (suite no-intermediates-internal ()
        (test invoke-later
          (piper 'nshnhi 'tdoo)
          1 (symbol?)
          (match? symbol=? 'irtiida)))))

  ;; When earlier returns multiple values, later accepts them.

  (test multiple-intermediates
    (pipe (lambda (num)
            (values (square num) (halve num)))
          expt)
    1 (procedure?)
    (lambda (piper)
      (suite multiple-intermediates-internal ()
        (test square-to-half-power
          (piper 543)
          1 (exact-integer?)
          (match? = (expt (square 543) (halve 543)))))))

  ;; When earlier is a variable-arity procedure, so is the piper.

  (test variable-arity-earlier
    (pipe + halve)
    1 (procedure?)
    (lambda (piper)
      (suite variable-arity-earlier-internal ()
        (test add-and-halve
          (piper 984 532 249 953 930 159 16)
          1 (exact-integer?)
          (match? = (halve (+ 984 532 249 953 930 159 16)))))))

  ;; When later is a variable-arity procedure, earlier can return
  ;; arbitrarily many values.

  (test variable-arity-later
    (pipe (lambda arguments (apply values (map square arguments)))
          +)
    1 (procedure?)
    (lambda (piper)
      (suite variable-arity-later-internal ()
        (test square-and-add
          (piper 968 113 333 179 655)
          1 (exact-integer?)
          (match? = (+ (square 968) (square 113) (square 333) (square 179)
                       (square 655))))))))

(suite transfer-unit ()
  (test simplest-case
    (transfer-unit 1 0)
    2 (exact-integer? exact-integer?)
    (match? = 0 1))
  (test exotic-case
    (transfer-unit -70/51 889/3)
    2 (exact-rational? exact-rational?)
    (match? = -121/51 892/3))
  (test text-example
    (transfer-unit 861 19)
    2 (exact-integer? exact-integer?)
    (match? = 860 20)))

(suite cross ()
  (test no-arguments
    (cross)
    1 (procedure?)
    (lambda (crosser)
      (suite no-arguments-internal ()
        (test do-nothing
          (crosser)
          0 ()))))
  (test several-arguments
    (cross square add1 double (sect expt 950 <>))
    1 (procedure?)
    (lambda (crosser)
      (suite several-arguments-internal ()
        (test diversity
          (crosser 76 453 236 807)
          4 (exact-integer? exact-integer? exact-integer? exact-integer?)
          (match? = (square 76) (add1 453) (double 236)
                    (expt 950 807))))))) 

(suite dispatch ()
  (test no-arguments
    (dispatch)
    1 (procedure?)
    (lambda (dispatcher)
      (suite no-arguments-internal ()
        (test do-nothing
          (dispatcher)
          0 ()))))
  (test several-unary-procedures
    (dispatch square add1 double)
    1 (procedure?)
    (lambda (dispatcher)
      (suite unary-procedures-internal ()
        (test square-add1-and-double
          (dispatcher 5)
          3 (exact-integer? exact-integer? exact-integer?)
          (match? = (square 5) (add1 5) (double 5))))))
  (test several-binary-procedures
    (dispatch expt < >)
    1 (procedure?)
    (lambda (dispatcher)
      (suite binary-procedures-internal ()
        (test do-it-all
          (dispatcher 247 964)
          3 (exact-integer? boolean? boolean?)
          (lambda (alpha beta gamma)
            (and (= alpha (expt 247 964))
                 (true? beta)
                 (not gamma)))))))
  (test several-nullary-procedures
    (dispatch (create 'cllisft) (create 'aiea) (create 'osien))
    1 (procedure?)
    (lambda (dispatcher)
      (suite nullary-procedures-internal ()
        (test fetch-symbols
          (dispatcher)
          3 (symbol? symbol? symbol?)
          (match? symbol=? 'cllisft 'aiea 'osien)))))
  (test variable-arity
    (dispatch + - *)
    1 (procedure?)
    (lambda (dispatcher)
      (suite variable-arity-internal ()
        (test three-ways
          (dispatcher 97 962 759 587 947 373)
          3 (exact-integer? exact-integer? exact-integer?)
          (match? = (+ 97 962 759 587 947 373)
                    (- 97 962 759 587 947 373)
                    (* 97 962 759 587 947 373))))))
  (test text-example
    (pipe (dispatch + *) <)
    1 (procedure?)
    (lambda (sum-less-than-product?)
      (suite text-example-internal ()
        (test no-arguments
          (sum-less-than-product?)
          1 (boolean?)
          true?)
        (test one-argument
          (sum-less-than-product? 890)
          1 (boolean?)
          not)
        (test multiple-arguments
          (sum-less-than-product? 1/4 3/5 2/7 113/355)
          1 (boolean?)
          not)))))

;;; Test the procedures in the "Adapters" section.

(suite >initial ()
  (test one-argument
    (>initial 'fonnrti)
    1 (symbol?)
    (match? symbol=? 'fonnrti))
  (test multi-argument
    (>initial 'erdior 'coatsnr 'gwtlrn 'osenr)
    1 (symbol?)
    (match? symbol=? 'erdior)))

(suite div ()
  (test positive-dividend-positive-divisor
    (div 123 10)
    1 (exact-integer?)
    (match? = 12))
  (test positive-dividend-positive-divisor-no-remainder
    (div 123 41)
    1 (exact-integer?)
    (match? = 3))
  (test zero-dividend-positive-divisor
    (div 0 10)
    1 (exact-integer?)
    zero?)
  (test negative-dividend-positive-divisor
    (div -123 10)
    1 (exact-integer?)
    (match? = -13))
  (test negative-dividend-positive-divisor-no-remainder
    (div -123 41)
    1 (exact-integer?)
    (match? = -3))
  (test zero-divisor
    (guard (raised-object
             ((error-object? raised-object) #t)
             (else (raise raised-object)))
      (div 123 0))
    1 (boolean?)
    true?)
  (test positive-dividend-negative-divisor
    (div 123 -10)
    1 (exact-integer?)
    (match? = -12))
  (test positive-dividend-negative-divisor-no-remainder
    (div 123 -41)
    1 (exact-integer?)
    (match? = -3))
  (test zero-dividend-negative-divisor
    (div 0 -10)
    1 (exact-integer?)
    zero?)
  (test negative-dividend-positive-divisor
    (div -123 -10)
    1 (exact-integer?)
    (match? = 13))
  (test negative-dividend-negative-divisor-no-remainder
    (div -123 -41)
    1 (exact-integer?)
    (match? = 3)))

(suite >next ()
  (test two-argument
    (>next 'dthhto 'iht)
    1 (symbol?)
    (match? symbol=? 'iht))
  (test multi-argument
    (>next 'elnhdh 'rssrvst 'htaea 'eemtc 'hto)
    1 (symbol?)
    (match? symbol=? 'rssrvst)))

(suite mod ()
  (test positive-dividend-positive-divisor
    (mod 123 10)
    1 (exact-integer?)
    (match? = 3))
  (test positive-dividend-positive-divisor-no-remainder
    (mod 123 41)
    1 (exact-integer?)
    zero?)
  (test zero-dividend-positive-divisor
    (mod 0 10)
    1 (exact-integer?)
    zero?)
  (test negative-dividend-positive-divisor
    (mod -123 10)
    1 (exact-integer?)
    (match? = 7))
  (test negative-dividend-positive-divisor-no-remainder
    (mod -123 41)
    1 (exact-integer?)
    zero?)
  (test zero-divisor
    (guard (raised-object
             ((error-object? raised-object) #t)
             (else (raise raised-object)))
      (mod 123 0))
    1 (boolean?)
    true?)
  (test positive-dividend-negative-divisor
    (mod 123 -10)
    1 (exact-integer?)
    (match? = 3))
  (test positive-dividend-negative-divisor-no-remainder
    (mod 123 -41)
    1 (exact-integer?)
    zero?)
  (test zero-dividend-negative-divisor
    (mod 0 -10)
    1 (exact-integer?)
    zero?)
  (test negative-dividend-positive-divisor
    (mod -123 -10)
    1 (exact-integer?)
    (match? = 7))
  (test negative-dividend-negative-divisor-no-remainder
    (mod -123 -41)
    1 (exact-integer?)
    zero?))

(suite >all-but-initial ()
  (test one-argument
    (>all-but-initial 'awti)
    0 ())
  (test multi-argument
    (>all-but-initial 'mieynal 'odld 'dhnns 'srnht 'eagmhoy)
    4 (symbol? symbol? symbol? symbol?)
    (match? symbol=? 'odld 'dhnns 'srnht 'eagmhoy)))

(suite identity ()
  (test does-it-work?
    (identity -13/4)
    1 (exact-rational?)
    (match? = -13/4)))

(suite >exch ()
  (test two-argument
    (>exch 'iogto 'aka)
    2 (symbol? symbol?)
    (match? symbol=? 'aka 'iogto))
  (test multi-argument
    (>exch 'lhse 'unhkame 'ksi 'hstsht)
    4 (symbol? symbol? symbol? symbol?)
    (match? symbol=? 'unhkame 'lhse 'ksi 'hstsht)))

(suite converse ()
  (test converse
    (converse expt) 
    1 (procedure?)
    (lambda (ponexentiate)
      (suite ponexentiate ()
        (test simple
          (ponexentiate 915 860)
          1 (exact-integer?)
          (match? = (expt 860 915))))))
  (test reverse-subtraction
    ((converse -) 7 5)
    1 (exact-integer?)
    (match? = -2)))

(suite ~initial ()
  (test square-tweak
    (~initial square)
    1 (procedure?)
    (lambda (tweaker)
      (suite square-~internal ()
        (test one-argument
          (tweaker 727)
          1 (exact-integer?)
          (match? = (square 727)))
        (test multi-argument
          (tweaker 744 502 106 818)
          4 (exact-integer? exact-integer? exact-integer? exact-integer?)
          (match? = (square 744) 502 106 818))))))

(suite ~next ()
  (test halve-tweaker
    (~next halve)
    1 (procedure?)
    (lambda (tweaker)
      (test two-argument
        (tweaker 354 174)
        2 (exact-integer? exact-integer?)
        (match? = 354 (halve 174)))
      (test multi-argument
        (tweaker 942 304 344 348)
        4 (exact-integer? exact-integer? exact-integer? exact-integer?)
        (match? =  942 (halve 304) 344 348)))))

(suite ~each ()
  (test generalized-square
    (~each square)
    1 (procedure?)
    (lambda (multi-squarer)
      (suite multi-squarer ()
        (test no-arguments
          (multi-squarer)
          0 ())
        (test several-arguments
          (multi-squarer 935 852 324 146 780 913 178)
          7 (exact-integer? exact-integer? exact-integer? exact-integer?
                            exact-integer? exact-integer? exact-integer?)
          (match? = (square 935) (square 852) (square 324) (square 146)
                    (square 780) (square 913) (square 178)))))))

(suite alternative-same-in-parity? ()
  (test both-even
    (same-in-parity? 420 306)
    1 (boolean?)
    true?)
  (test one-even
    (same-in-parity? 259 730)
    1 (boolean?)
    not)
  (test neither-even
    (same-in-parity? 821 31)
    1 (boolean?)
    true?))

(suite compare-by ()
  (test smallness
    (compare-by square <=)
    1 (procedure?)
    (lambda (nonshrinking?)
      (suite smallness-internal ()
        (test smaller
          (nonshrinking? 275 -915)
          1 (boolean?)
          true?)
        (test equally-small
          (nonshrinking? -367 367)
          1 (boolean?)
          true?)
        (test no
          (nonshrinking? 426 169)
          1 (boolean?)
          not)
        (test more-arguments-yes
          (nonshrinking? 15 -612 661 -811 -825)
          1 (boolean?)
          true?)
        (test more-arguments-no
          (nonshrinking? 875 829 291 204 629 635)
          1 (boolean?)
          not)))))

(suite same-in-parity? ()
  (test both-even
    (same-in-parity? 326 506)
    1 (boolean?)
    true?)
  (test one-even
    (same-in-parity? 961 376)
    1 (boolean?)
    not)
  (test neither-even
    (same-in-parity? 571 439)
    1 (boolean?)
    true?))

;;; Test the procedures in the "Recursion managers" section.

(suite recur ((least-divisor
               (lambda (num)
                 ((recur (pipe (sect mod num <>) zero?)
                         identity
                         (dispatch identity add1)
                         >next) 2))))
  (test small-case
    (least-divisor 2)
    1 (exact-integer?)
    (match? = 2))
  (test composite
    (least-divisor (* 89 113))
    1 (exact-integer?)
    (match? = 89))
  (test large-prime
    (least-divisor 1009)
    1 (exact-integer?)
    (match? = 1009))

 ;; Make sure that the minimal, do-nothing recursion works.

  (test do-nothing
    (recur (constant #t) black-hole (constant #f #f) black-hole)
    1 (procedure?)
    (lambda (recurrer)
      (test do-nothing-recursively
        (recurrer 'oan)
        0 ())))

  ;; Define a procedure that finds the sum of the prime divisors of a
  ;; natural number by descending recursively, dividing out one prime
  ;; divisor at each step.

  (test sum-of-prime-divisors
    (recur (sect = <> 1)
           (constant 0)
           (pipe (dispatch identity least-divisor) (dispatch >next div))
           +)
    1 (procedure?)
    (lambda (recurrer)
      (suite sum-of-prime-divisors-internal ()
        (test run-it
          (recurrer 392)
          1 (exact-integer?)
          (match? = 20)))))

  ;; 392 = 2 * 2 * 2 * 7 * 7 and 20 = 2 + 2 + 2 + 7 + 7.

  ;; This artificial case carries along more values.

  (test more-values
    (recur zero?
           (constant 0 0 0 0 0)
           (dispatch identity sub1)
           (lambda (counter alpha beta gamma delta epsilon)
             (values (+ counter alpha)
                     (+ counter alpha beta)
                     (+ counter alpha beta gamma)
                     (+ counter alpha beta gamma delta)
                     (+ counter alpha beta gamma delta epsilon))))
    1 (procedure?)
    (lambda (recurrer)
      (suite more-values-internal ()
        (test does-it-work?
          (recurrer 7)
          5 (exact-integer? exact-integer? exact-integer? exact-integer?
                            exact-integer?)
          (match? = 28 84 210 462 924))))))

(suite build ()
  (test minimal
    (build values?
           (create)
           (create)
           black-hole
           (create))
    1 (procedure?)
    (lambda (stopper)
      (suite minimal-internal ()
        (test stop-at-once
          (stopper)
          0 ()))))
  (test factorial
    (build zero? (constant 1) identity sub1 *)
    1 (procedure?)
    (lambda (fact)
      (suite factorial-internal ()
        (test zero
          (fact 0)
          1 (exact-integer?)
          (match? = 1))
        (test higher
          (fact 789)
          1 (exact-integer?)
          (match? = (factorial 789))))))
  (test multi-argument
    (build (pipe >next zero?)
           >initial
           (constant 'foo)
           (cross add1 sub1)
           >next)
    1 (procedure?)
    (lambda (builder)
      (suite multi-argument-internal ()
        (test zero-augend
          (builder 0 218)
          1 (exact-integer?)
          (match? = 218))
        (test higher-augend
          (builder 156 649)
          1 (exact-integer?)
          (match? = (+ 156 649))))))
  (test multi-result
    (build (pipe >next zero?)
           values
           -
           (cross add1 sub1)
           (lambda (diff left right)
             (values diff (+ left right))))
    1 (procedure?)
    (lambda (builder)
      (suite multi-result-internal ()
        (test zero-counter
          (builder 449 0)
          2 (exact-integer? exact-integer?)
          (match? = 449 0))
        (test larger-counter
          (builder 750 4)
          2 (exact-integer? exact-integer?)
          (match? = 746 3004))))))

(suite halve ()
  (test odd
    (halve -53)
    1 (exact-integer?)
    (match? = -27))
  (test zero
    (halve 0)
    1 (exact-integer?)
    zero?)
  (test even
    (halve 182)
    1 (exact-integer?)
    (match? = 91)))

(suite power-of-two? ()
  (test unity
    (power-of-two? 1)
    1 (boolean?)
    true?)
  (test large-power-of-two
    (power-of-two? (expt 2 125))
    1 (boolean?)
    true?)
  (test near-large-power-of-two
    (power-of-two? (- (expt 2 247) 1))
    1 (boolean?)
    not)
  (test one-odd-factor
    (power-of-two? (* 3 (expt 2 464)))
    1 (boolean?)
    not)

  ;; Similar tests for the alternative definition.

  (test alternative-unity
    (alternative-power-of-two? 1)
    1 (boolean?)
    true?)
  (test alternative-large-power-of-two
    (alternative-power-of-two? (expt 2 331))
    1 (boolean?)
    true?)
  (test alternative-near-large-power-of-two
    (alternative-power-of-two? (- (expt 2 975) 1))
    1 (boolean?)
    not)
  (test alternative-one-odd-factor
    (alternative-power-of-two? (* 3 (expt 2 69)))
    1 (boolean?)
    not))

(suite check ()
  (test common-multiple
    (check (pipe >next zero?)
           (pipe mod zero?)
           (~next sub1))
  1 (procedure?)
  (lambda (checker)
    (suite common-multiple-internal ()
      (test zero-limit
        (checker 881 0)
        1 (boolean?)
        true?)
      (test zero-candidate
        (checker 0 834)
        1 (boolean?)
        true?)
      (test normal-affirmative
        (checker 83160 12)
        1 (boolean?)
        true?)
      (test normal-negative
        (checker 4860 9)
        1 (boolean?)
        not)))))

(suite iterate ()
  (test minimal
    (iterate values? (create))
    1 (procedure?)
    (lambda (stopper)
      (suite minimal-iterator ()
        (test stopper
          (stopper)
          0 ()))))
  (test countdown
    (iterate zero? (sect - <> 1))
    1 (procedure?)
    (lambda (counter-down)
      (suite countdown-internal ()
        (test does-it-go?
          (counter-down 659)
          1 (exact-integer?)
          zero?))))
  (test multiargument
    (iterate (pipe >next zero?)
             (cross add1 sub1))
    1 (procedure?)
    (lambda (adder)
      (suite multi-argument-internal ()
        (test zero-augend
          (adder 0 710)
          2 (exact-integer? exact-integer?)
          (match? = 710 0))
        (test zero-addend
          (adder 329 0)
          2 (exact-integer? exact-integer?)
          (match? = 329 0))
        (test general
          (adder 544 366)
          2 (exact-integer? exact-integer?)
          (match? = (+ 544 366) 0)))))
  (test Euclidean-gcd
    (pipe (iterate (pipe >next zero?) (dispatch >next mod)) >initial)
    1 (procedure?)
    (lambda (iterator)
      (suite Euclidean-gcd-internal ()
        (test Euclid-wins
          (iterator 338390548 378586796)
          1 (exact-integer?)
          (match? = (gcd 338390548 378586796))))))
  (test peasant-multiplication
    (iterate (lambda (so-far multiplicand multiplier)
               (zero? multiplier))
             (lambda (so-far multiplicand multiplier)
               (values (if (odd? multiplier)
                           (+ so-far multiplicand)
                           so-far)
                       (double multiplicand)
                       (halve multiplier))))
    1 (procedure?)
    (lambda (iterator)
      (suite peasant-multiplication-internal
          ((multiply (pipe (lambda (multiplicand multiplier)
                             (values 0 multiplicand multiplier))
                           (pipe iterator >initial))))
        (test does-it-work?
          (multiply 156744798 744493571)
          1 (exact-integer?)
          (match? = (* 156744798 744493571)))))))

(suite greatest-odd-divisor ()
  (test halve-until-odd
    (iterate odd? halve)
    1 (procedure?)
    (lambda (iterator)
      (suite halve-until-odd-internal ()
        (test no-halving
          (iterator 865)
          1 (exact-integer?)
          (match? = 865))
        (test halve-once
          (iterator 6)
          1 (exact-integer?)
          (match? = 3))
        (test halve-several-times
          (iterator 200)
          1 (exact-integer?)
          (match? = 25))))))

(suite ceiling-of-log-two ()
  (test one
    (ceiling-of-log-two 1)
    1 (exact-integer?)
    zero?)
  (test two
    (ceiling-of-log-two 2)
    1 (exact-integer?)
    (match? = 1))
  (test three
    (ceiling-of-log-two 3)
    1 (exact-integer?)
    (match? = 2))
  (test four
    (ceiling-of-log-two 4)
    1 (exact-integer?)
    (match? = 2))
  (test just-short
    (ceiling-of-log-two (- (expt 2 51) 1))
    1 (exact-integer?)
    (match? = 51))
  (test exact-power
    (ceiling-of-log-two (expt 2 719))
    1 (exact-integer?)
    (match? = 719))
  (test just-over
    (ceiling-of-log-two (+ (expt 2 923) 1))
    1 (exact-integer?)
    (match? = 924)))

(suite alternative-ceiling-of-log-two ()
  (test one
    (alternative-ceiling-of-log-two 1)
    1 (exact-integer?)
    zero?)
  (test two
    (alternative-ceiling-of-log-two 2)
    1 (exact-integer?)
    (match? = 1))
  (test three
    (alternative-ceiling-of-log-two 3)
    1 (exact-integer?)
    (match? = 2))
  (test four
    (alternative-ceiling-of-log-two 4)
    1 (exact-integer?)
    (match? = 2))
  (test just-short
    (alternative-ceiling-of-log-two (- (expt 2 491) 1))
    1 (exact-integer?)
    (match? = 491))
  (test exact-power
    (alternative-ceiling-of-log-two (expt 2 673))
    1 (exact-integer?)
    (match? = 673))
  (test just-over
    (alternative-ceiling-of-log-two (+ (expt 2 545) 1))
    1 (exact-integer?)
    (match? = 546)))

(suite conventional-ceiling-of-log-two ()
  (test one
    (conventional-ceiling-of-log-two 1)
    1 (exact-integer?)
    zero?)
  (test two
    (conventional-ceiling-of-log-two 2)
    1 (exact-integer?)
    (match? = 1))
  (test three
    (conventional-ceiling-of-log-two 3)
    1 (exact-integer?)
    (match? = 2))
  (test four
    (conventional-ceiling-of-log-two 4)
    1 (exact-integer?)
    (match? = 2))
  (test just-short
    (conventional-ceiling-of-log-two (- (expt 2 719) 1))
    1 (exact-integer?)
    (match? = 719))
  (test exact-power
    (conventional-ceiling-of-log-two (expt 2 923))
    1 (exact-integer?)
    (match? = 923))
  (test just-over
    (conventional-ceiling-of-log-two (+ (expt 2 511) 1))
    1 (exact-integer?)
    (match? = 512)))

;;; Test the procedures in the "Euclid's algorithm" section.

(suite divisible-by? ()
  (test zero
    (divisible-by? 0 -612)
    1 (boolean?)
    true?)
  (test self
    (divisible-by? -993 -993)
    1 (boolean?)
    true?)
  (test multiple
    (divisible-by? (* -443 457) -443)
    1 (boolean?)
    true?)
  (test does-not-evenly-divide
    (divisible-by? 676950 421)
    1 (boolean?)
    not))

(suite brute-force-greatest-common-divisor ()

  ;; Since the positive integers have no upper bound, the only boundary
  ;; cases to check are the ones at the small end:

  (test least-distinct-numbers
    (brute-force-greatest-common-divisor 2 1)
    1 (exact-integer?)
    (match? = 1))

  (test least-distinct-non-units
    (brute-force-greatest-common-divisor 3 2)
    1 (exact-integer?)
    (match? = 1))

  ;; Perhaps it's worth while to check the first case in which the given
  ;; numbers are not relatively prime and neither divides the other.

  (test least-non-relatively-prime-neither-dividing-other
    (brute-force-greatest-common-divisor 6 4)
    1 (exact-integer?)
    (match? = 2))

  ;; Here are a couple more cases in which I happened to know the answer:

  (test text-case
    (brute-force-greatest-common-divisor 42 30)
    1 (exact-integer?)
    (match? = 6))

  (test draft-case
    (brute-force-greatest-common-divisor 101135853 5133)
    1 (exact-integer?)
    (match? = 177))

  ;; The text claims that the code works even when the first argument is
  ;; less than the second, provided both are positive.  Let's check this:

  (test first-argument-less
    (brute-force-greatest-common-divisor 3192 3444)
    1 (exact-integer?)
    (match? = 84))

  ;; The text also claims that the code works when the arguments are equal.

  (test minimal-equal-arguments
    (brute-force-greatest-common-divisor 1 1)
    1 (exact-integer?)
    (match? = 1))

  (test equal-arguments
    (brute-force-greatest-common-divisor 98 98)
    1 (exact-integer?)
    (match? = 98)))

(suite greater-and-lesser ()
  (test left-lesser
    (greater-and-lesser -95/478 914/500)
    2 (exact-rational? exact-rational?)
    (match? = 914/500 -95/478))
  (test right-lesser
    (greater-and-lesser 569 552)
    2 (exact-integer? exact-integer?)
    (match? = 569 552))
  (test equal
    (greater-and-lesser 996/532 249/133)
    2 (exact-rational? exact-rational?)
    (match? = 498/266 498/266)))

(suite greatest-common-divisor ()

  ;; Since the positive integers have no upper bound, the only boundary
  ;; cases to check are the ones at the small end:

  (test least-distinct-numbers
    (greatest-common-divisor 2 1)
    1 (exact-integer?)
    (match? = 1))

  (test least-distinct-non-units
    (greatest-common-divisor 3 2)
    1 (exact-integer?)
    (match? = 1))

  ;; Perhaps it's worth while to check the first case in which the given
  ;; numbers are not relatively prime and neither divides the other.

  (test least-non-relatively-prime-neither-dividing-other
    (greatest-common-divisor 6 4)
    1 (exact-integer?)
    (match? = 2))

  ;; Here's a case involving very large numbers in which it's not hard to
  ;; recognize whether the result is correct:

  (test large-numbers
    (greatest-common-divisor (expt 10 61) (expt 6 61))
    1 (exact-integer?)
    (match? = (expt 2 61)))

  ;; Here are a couple more cases in which I happened to know the answer:

  (test text-case
    (greatest-common-divisor 42 30)
    1 (exact-integer?)
    (match? = 6))

  (test draft-case
    (greatest-common-divisor 101135853 45014640)
    1 (exact-integer?)
    (match? = 177))

  ;; The text claims that the code works even when the first argument is
  ;; less than the second, provided both are positive.  Let's check this:

  (test first-argument-less
    (greatest-common-divisor 3192 3444)
    1 (exact-integer?)
    (match? = 84))

  ;; The text also claims that the code works when the arguments are equal.

  (test minimal-equal-arguments
    (greatest-common-divisor 1 1)
    1 (exact-integer?)
    (match? = 1))

  (test equal-arguments
    (greatest-common-divisor 98 98)
    1 (exact-integer?)
    (match? = 98)))

;;; Test the procedures from the "Raised Boolean procedures" section.

(suite ^not ()
  (test nullary-predicate
    (^not (create #f))
    1 (procedure?)
    (lambda (complement)
      (suite nullary-predicate-internal ()
        (test it-is-true
          (complement)
          1 (boolean?)
          true?))))
  (test non-zero?  ; example from the text
    (^not zero?)
    1 (procedure?)
    (lambda (non-zero?)
      (suite non-zero?-internal ()
        (test zero
          (non-zero? 0)
          1 (boolean?)
          not)
        (test other
          (non-zero? 433)
          1 (boolean?)
          true?))))
  (test uneven?  ; example from the text
    (^not even?)
    1 (procedure?)
    (lambda (uneven?)
      (suite uneven?-internal ()
        (test even
          (uneven? 788)
          1 (boolean?)
          not)
        (test odd
          (uneven? 453)
          1 (boolean?)
          true?))))
  (test non-ascending?
    (^not <)
    1 (procedure?)
    (lambda (non-ascending?)
      (suite non-ascending?-internal ()
        (test up
          (non-ascending? 716 722 927 959)
          1 (boolean?)
          not)
        (test down
          (non-ascending? 762 653 444 108)
          1 (boolean?)
          true?)

        ;; Note that (^not <) is not the same as >= if there are more than
        ;; two arguments!

        (test wobbly
          (non-ascending? 178 565 480 726)
          1 (boolean?)
          true?)))))

(suite ^et ()
  (test nullary-predicates
    (^et (create #t) (create #f))
    1 (procedure?)
    (lambda (conjoined)
      (suite nullary-predicates-internal ()
        (test it-is-false
          (conjoined)
          1 (boolean?)
          not))))
  (test really-special?
    (^et (sect divisible-by? <> 3) odd?)
    1 (procedure?)
    (lambda (conjoined)
      (suite really-special?-internal ()
        (test both-satisfied
          (conjoined 15)
          1 (boolean?)
          true?)
        (test left-satisfied
          (conjoined 192)
          1 (boolean?)
          not)
        (test right-satisfied
          (conjoined 889)
          1 (boolean?)
          not)
        (test neither-satisfied
          (conjoined 40)
          1 (boolean?)
          not))))
  (test Pythagorean?
    (^et < (lambda (short-leg long-leg hypotenuse)
             (= (+ (square short-leg) (square long-leg))
                (square hypotenuse))))
    1 (procedure?)
    (lambda (conjoined)
      (suite Pythagorean?-internal ()
        (test unlikely
          (conjoined 188 635 663)
          1 (boolean?)
          not)
        (test better
          (conjoined 133 156 205)
          1 (boolean?)
          true?)))))

(suite ^vel ()
  (test nullary-predicates
    (^vel (create #t) (create #f))
    1 (procedure?)
    (lambda (disjoined)
      (suite nullary-predicates-internal ()
       (test it-is-true
         (disjoined)
         1 (boolean?)
         true?))))
  (test special?
    (^vel (sect divisible-by? <> 3) odd?)
    1 (procedure?)
    (lambda (disjoined)
      (suite special?-internal ()
        (test both-satisfied
          (disjoined 249)
          1 (boolean?)
          true?)
        (test left-satisfied
          (disjoined 276)
          1 (boolean?)
          true?)
        (test right-satisfied
          (disjoined 17)
          1 (boolean?)
          true?)
        (test neither-satisfied
          (disjoined 758)
          1 (boolean?)
          not))))
  (test monotonic?
    (^vel < >)
    1 (procedure?)
    (lambda (disjoined)
      (suite monotonic?-internal ()
        (test ascending
          (disjoined 219 312 488 698)
          1 (boolean?)
          true?)
        (test descending
          (disjoined 805 534 471 153 53)
          1 (boolean?)
          true?)
        (test wobbly
          (disjoined 54 1 232)
          1 (boolean?)
          not)))))

(suite ^if ()
  (test do-nothing
    (^if (create #t) (create) (create))
    1 (procedure?)
    (lambda (iffer)
      (suite do-nothing-internal ()
        (test do-it-elegantly
          (iffer)
          0 ()))))
  (test square-or-halve
    (^if even? square halve)
    1 (procedure?)
    (lambda (iffer)
      (suite square-or-halve-internal ()
        (test square-on-even
          (iffer 550)
          1 (exact-integer?)
          (match? = (square 550)))
        (test halve-on-odd
          (iffer 257)
          1 (exact-integer?)
          (match? = (halve 257))))))
  (test variable-arity
    (^if = + -)
    1 (procedure?)
    (lambda (iffer)
      (suite variable-arity-internal ()
        (test all-equal
          (iffer 173 173 173)
          1 (exact-integer?)
          (match? = (+ 173 173 173)))
        (test unequal
          (iffer 383 220 959 889 826)
          1 (exact-integer?)
          (match? = (- 383 220 959 889 826)))))))

(suite disparity ()
  (test equal-arguments
    (disparity 231 231)
    1 (exact-integer?)
    zero?)
  (test greater-first
    (disparity 616 551)
    1 (exact-integer?)
    (match? = (- 616 551)))
  (test lesser-first
    (disparity 588 920)
    1 (exact-integer?)
    (match? = (- 920 588)))
  (test different-signs
    (disparity 589 -457)
    1 (exact-integer?)
    (match? = (- 589 -457))))

(suite conditionally-combine ()
  (test unary-combiner
    (conditionally-combine even? (lambda (ignored) (values)))
    1 (procedure?)
    (lambda (ignorer)
      (suite unary-combiner-internal ()
        (test even
          (ignorer 362)
          0 ())
        (test odd
          (ignorer 361)
          0 ()))))
  (test binary-combiner
    (conditionally-combine even? -)
    1 (procedure?)
    (lambda (proc)
      (suite binary-combiner-internal ()
        (test even
          (proc 234 184)
          1 (exact-integer?)
          (match? = 50))
        (test odd
          (proc 941 872)
          1 (exact-integer?)
          (match? =  872)))))
  (test greater-arity-combiner
    (conditionally-combine even? (lambda (alpha beta gamma delta)
                                   (values (- alpha beta)
                                           (- beta gamma)
                                           (- gamma delta))))
    1 (procedure?)
    (lambda (proc)
      (suite greater-arity-combiner-internal ()
        (test even
          (proc 872 941 733 777)
          3 (exact-integer? exact-integer? exact-integer?)
          (match? = (- 872 941) (- 941 733) (- 733 777)))
        (test odd
          (proc 973 965 86 764)
          3 (exact-integer? exact-integer? exact-integer?)
          (match? = 965 86 764))))))

;;; Test the procedures from the "Natural numbers and recursion" section.

(suite natural-mapper ((base (create 1))
                       (step double))
  (test text-example
    (rec (natural-mapper nat)
      (if (zero? nat)
          (base)
          (receive recursive-results (natural-mapper (sub1 nat))
            (apply step recursive-results))))
    1 (procedure?)
    (lambda (power-of-two)
      (suite natural-mapper-internal ()
        (test zero
          (power-of-two 0)
          1 (exact-integer?)
          (match? = 1))
        (test large-integer
          (power-of-two 93)
          1 (exact-integer?)
          (match? = (expt 2 93)))))))

(suite fold-natural ()
  (test do-nothing
    (fold-natural (create) (create))
    1 (procedure?)
    (lambda (do-nothing)
      (suite do-nothing-internal ()
        (test do-nothing-gracefully
          (do-nothing 466)
          0 ()))))
  (test power-of-two-by-doubling
    (fold-natural (create 1) double)
    1 (procedure?)
    (lambda (power-of-two)
      (suite power-of-two-by-doubling-internal ()
        (test zero
          (power-of-two 0)
          1 (exact-integer?)
          (match? = 1))
        (test positive-power
          (power-of-two 7)
          1 (exact-integer?)
          (match? = 128)))))

  ;; The following artificial example shows how fold-natural works when
  ;; there are several state values.

  (test extended-Fibonacci
    (fold-natural (create 0 0 0 0 1)
                  (lambda (unua dua tria kvara kvina)
                    (values dua tria kvara kvina (+ unua dua tria kvara
                                                    kvina))))
    1 (procedure?)
    (lambda (folder)
      (suite extended-Fibonacci-internal ()
        (test does-it-work?
          (folder 17)
          5 (natural-number? natural-number? natural-number?
                             natural-number? natural-number?)
          (match? = 3525 6930 13624 26784 52656)))))

  ;; Now we repeat the tests on the alternative definition.

  (test alternative-do-nothing
    (alternative-fold-natural (create) (create))
    1 (procedure?)
    (lambda (do-nothing)
      (suite alternative-do-nothing-internal ()
        (test do-nothing-gracefully
          (do-nothing 466)
          0 ()))))
  (test alternative-power-of-two-by-doubling
    (alternative-fold-natural (create 1) double)
    1 (procedure?)
    (lambda (power-of-two)
      (suite alternative-power-of-two-by-doubling-internal ()
        (test zero
          (power-of-two 0)
          1 (exact-integer?)
          (match? = 1))
        (test positive-power
          (power-of-two 7)
          1 (exact-integer?)
          (match? = 128)))))
  (test alternative-extended-Fibonacci
    (fold-natural (create 0 0 0 0 1)
                  (lambda (unua dua tria kvara kvina)
                    (values dua tria kvara kvina (+ unua dua tria kvara
                                                    kvina))))
    1 (procedure?)
    (lambda (folder)
      (suite alternative-extended-Fibonacci-internal ()
        (test does-it-work?
          (folder 17)
          5 (natural-number? natural-number? natural-number?
                             natural-number? natural-number?)
          (match? = 3525 6930 13624 26784 52656))))))

(suite ply-natural ()
  (test do-nothing
    (ply-natural (create) black-hole)
    1 (procedure?)
    (lambda (idler)
      (suite do-nothing-internal ()
        (test do-nothing-gracefully
          (idler 0)
          0 ())
        (test do-nothing-gracefully-at-length
          (idler 867)
          0 ()))))
  (test factorial  ; example from the text
    (ply-natural (create 1) *)
    1 (procedure?)
    (lambda (concise-factorial)
      (suite factorial-internal ()
        (test small
          (concise-factorial 5)
          1 (exact-integer?)
          (match? = (factorial 5)))
        (test large
          (concise-factorial 122)
          1 (exact-integer?)
          (match? = (factorial 122)))
        (test zero
          (concise-factorial 0)
          1 (exact-integer?)
          (match? = 1)))))

  ;; The following artificial example shows how ply-natural works when
  ;; there are several ``state'' arguments.

  (test Pascal-triangle
    (ply-natural (create 0 0 0 0)
                 (lambda (guide unua dua tria kvara)
                   (values (+ guide unua)
                           (+ unua dua)
                           (+ dua tria)
                           (+ tria kvara))))
    1 (procedure?)
    (lambda (chooser)
      (suite Pascal-triangle-internal ()
        (test run-it
          (chooser 17)
          4 (natural-number? natural-number? natural-number?
             natural-number?)
          (match? = 153 816 3060 8568))))))

(suite lower-ply-natural ()
  (test do-nothing
    (lower-ply-natural (create) black-hole)
    1 (procedure?)
    (lambda (idler)
      (suite do-nothing-internal ()
        (test do-nothing-gracefully
          (idler 0)
          0 ())
        (test do-nothing-at-length
          (idler 49)
          0 ()))))
  (test subtermial
    (lower-ply-natural (create 0) +)
    1 (procedure?)
    (lambda (subtermial)
      (suite subtermial-internal ()
        (test zero
          (subtermial 0)
          1 (natural-number?)
          zero?)
        (test one
          (subtermial 1)
          1 (natural-number?)
          zero?)
        (test typical
          (subtermial 562)
          1 (natural-number?)
          (match? = 157641)))))

  ;; The following artificial example shows how lower-ply-natural works
  ;; when there are several ``state'' arguments.

  (test Pascal-triangle
    (lower-ply-natural (create 0 0 0 0)
                       (lambda (guide unua dua tria kvara)
                         (values (+ guide unua)
                                 (+ unua dua)
                                 (+ dua tria)
                                 (+ tria kvara))))
    1 (procedure?)
    (lambda (chooser)
      (suite Pascal-triangle-internal ()
        (test run-it
          (chooser 99)
          4 (natural-number? natural-number? natural-number?
             natural-number?)
          (match? = 4851 156849 3764376 71523144))))))

(suite summation ()
  (test singleton-range
    (summation square 330 330)
    1 (exact-integer?)
    (match? = (square 330)))
  (test larger-range
    (summation factorial 56 62)
    1 (exact-integer?)
    (match? = (+ (factorial 56) (factorial 57) (factorial 58)
                 (factorial 59) (factorial 60) (factorial 61)
                 (factorial 62))))
  (test alternative-singleton-range
    (alternative-summation square 434 434)
    1 (exact-integer?)
    (match? = (square 434)))
  (test alternative-larger-range
    (alternative-summation factorial 72 76)
    1 (exact-integer?)
    (match? = (+ (factorial 72) (factorial 73) (factorial 74)
                 (factorial 75) (factorial 76)))))

(suite tally ()
  (test always-final
    (tally (create #t) black-hole)
    1 (procedure?)
    (lambda (optimist)
      (suite always-final-internal ()
        (test try-it
          (optimist)
          1 (natural-number?)
          zero?))))
  (test countdown
    (tally zero? sub1)
    1 (procedure?)
    (lambda (tallier)
      (suite countdown-internal ()
        (test immediate-blastoff
          (tallier 0)
          1 (natural-number?)
          zero?)
        (test long-count
          (tallier 506)
          1 (natural-number?)
          (match? = 506)))))
  (test multi-argument
    (tally = (~initial sub1))
    1 (procedure?)
    (lambda (minus)
      (suite multi-argument-internal ()
        (test zero-subtrahend
          (minus 391 0)
          1 (natural-number?)
          (match? = 391))
        (test equal-operands
          (minus 386 386)
          1 (natural-number?)
          zero?)
        (test general-case
          (minus 450 57)
          1 (natural-number?)
          (match? = (- 450 57))))))

  ;; Similar tests for the alternative definition.

  (test alternative-always-final
    (alternative-tally (create #t) black-hole)
    1 (procedure?)
    (lambda (optimist)
      (suite always-final-internal ()
        (test try-it
          (optimist)
          1 (natural-number?)
          zero?))))
  (test alternative-countdown
    (alternative-tally zero? sub1)
    1 (procedure?)
    (lambda (tallier)
      (suite countdown-internal ()
        (test immediate-blastoff
          (tallier 0)
          1 (natural-number?)
          zero?)
        (test long-count
          (tallier 639)
          1 (natural-number?)
          (match? = 639)))))
  (test alternative-multi-argument
    (alternative-tally = (~initial sub1))
    1 (procedure?)
    (lambda (minus)
      (suite multi-argument-internal ()
        (test zero-subtrahend
          (minus 694 0)
          1 (natural-number?)
          (match? = 694))
        (test equal-operands
          (minus 829 829)
          1 (natural-number?)
          zero?)
        (test general-case
          (minus 111 10)
          1 (natural-number?)
          (match? = (- 111 10)))))))

(suite ceiling-of-log-two-using-tally ()
  (test one
    (ceiling-of-log-two-using-tally 1)
    1 (exact-integer?)
    zero?)
  (test two
    (ceiling-of-log-two-using-tally 2)
    1 (exact-integer?)
    (match? = 1))
  (test three
    (ceiling-of-log-two-using-tally 3)
    1 (exact-integer?)
    (match? = 2))
  (test four
    (ceiling-of-log-two-using-tally 4)
    1 (exact-integer?)
    (match? = 2))

  ;; If the bound is just short of a large power of two, the result should
  ;; be the exponent of that power.

  (test just-short
    (ceiling-of-log-two-using-tally (- (expt 2 157) 1))
    1 (exact-integer?)
    (match? = 157))

  ;; If the bound is a large power of two, the result should be the
  ;; exponent of that power.

  (test just-short
    (ceiling-of-log-two-using-tally (expt 2 535))
    1 (exact-integer?)
    (match? = 535))

  ;; If the bound is just above a large power of two, the result should be
  ;; the successor of the exponent of that power.

  (test just-over
    (ceiling-of-log-two-using-tally (+ (expt 2 965) 1))
    1 (exact-integer?)
    (match? = 966)))

(suite conditionally-tally ()
  (test always-final
    (conditionally-tally values? values? black-hole)
    1 (procedure?)
    (lambda (optimist)
      (suite always-final-internal ()
        (test no-way
          (optimist)
          1 (natural-number?)
          zero?))))
  (test count-odds
    (conditionally-tally zero? odd? sub1)
    1 (procedure?)
    (lambda (countdown)
      (suite count-odds-internal ()
        (test zero
          (countdown 0)
          1 (natural-number?)
          zero?)
        (test larger
          (countdown 456)
          1 (natural-number?)
          (match? = 228)))))
  (test never-satisfied
    (conditionally-tally zero? (constant #f) sub1)
    1 (procedure?)
    (lambda (countdown)
      (suite never-satisfied-internal ()
        (test hapless
          (countdown 766)
          1 (natural-number?)
          zero?)))))

(suite for-all-less-than ()
  (test zero-only
    (for-all-less-than (constant #f))
    1 (procedure?)
    (lambda (spurner?)
      (suite zero-only-internal ()
        (test zero
          (spurner? 0)
          1 (boolean?)
          true?)
        (test positive
          (spurner? 562)
          1 (boolean?)
          not))))
  (test edge-cases
    (for-all-less-than (sect < <> 625))
    1 (procedure?)
    (lambda (below-edge?)
      (suite edge-cases-internal ()
        (test well-below-edge
          (below-edge? 624)
          1 (boolean?)
          true?)
        (test below-edge
          (below-edge? 625)
          1 (boolean?)
          true?)
        (test over-the-edge
          (below-edge? 626)
          1 (boolean?)
          not)
        (test mid-run
          (below-edge? 878)
          1 (boolean?)
          not)))))

(suite exists-less-than ()
  (test all-but-zero
    (exists-less-than values?)
    1 (procedure?)
    (lambda (trying?)
      (suite all-but-zero-internal ()
        (test zero
          (trying? 0)
          1 (boolean?)
          not)
        (test positive
          (trying? 289)
          1 (boolean?)
          true?))))
  (test edge-cases
    (exists-less-than (sect <= 932 <>))
    1 (procedure?)
    (lambda (above-edge?)
      (suite edge-cases-internal ()
        (test well-above-edge
          (above-edge? 931)
          1 (boolean?)
          not)
        (test above-edge

          (above-edge? 932)
          1 (boolean?)
          not)
        (test over-the-edge
          (above-edge? 933)
          1 (boolean?)
          true?)
        (test mid-run
          (above-edge? 964)
          1 (boolean?)
          true?)))))

;;; copyright (C) 2011, 2016 John David Stone

;;; This program is free software.  You may redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation -- either version 3 of the License, or (at
;;; your option) any later version.  A copy of the GNU General Public
;;; License is available on the World Wide Web at
;;;
;;;                http://www.gnu.org/licenses/gpl.html
;;;
;;; This program is distributed in the hope that it will be useful, but
;;; WITHOUT ANY WARRANTY -- without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;;; General Public License for more details.
